<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Events\AfterSheet;

use App\Models\Accounting;
use App\Models\Financialyear;

use DB;

class TrialBalanceExport implements FromCollection, WithHeadings, WithTitle, WithStyles, WithEvents
{
    protected $startDate;
    protected $endDate;
    protected $accountId;
    protected $fsDate;
    protected $feDate;
    protected $exTotal;
    protected $inTotal;
        
    // Accept start and end dates in the constructor
    public function __construct($startDate, $endDate, $accountId)
    {
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        $this->accountId = $accountId;
        $this->drTotal = 0;
        $this->crTotal = 0;
        
        $this->fyearId = Financialyear::getDefaultFyearId($this->accountId);
        $fyear = Financialyear::find($this->fyearId);
        if($fyear){
            $this->fsDate = $fyear->sdate;
            $this->feDate = $fyear->edate;
        }
    }
    
    /**
    * @return \Illuminate\Support\Collection
    */
    public function collection()
    {
        $liability_count = Accounting::getLedgers("liability")->count();
        $liabilities = Accounting::voucherAmount("liability",$this->accountId,$this->endDate,0,$liability_count);
            
        $asset_count = Accounting::getLedgers("asset")->count();
        $assets = Accounting::voucherAmount("asset",$this->accountId,$this->endDate,0,$asset_count);
            
        $expense_count = Accounting::getLedgers("expense")->count();
        $expenses = Accounting::profitLoss("expense",$this->accountId,$this->endDate,'',0,$expense_count);
            
        $income_count = Accounting::getLedgers("income")->count();
        $incomes = Accounting::profitLoss("income",$this->accountId,$this->endDate,'',0,$income_count);
        
        $profit_loss_cr=$profit_loss_dr=0;
        $profit_loss = DB::table('ledger_total_bal')->sum('ledger_amount');
        if($profit_loss > 0){
            $profit_loss_cr = $profit_loss;
        }else{
            $profit_loss_dr = abs($profit_loss);
        }
        
        $this->drTotal += $profit_loss_dr;
        $this->crTotal += $profit_loss_cr;
        
        $result = [];
        
        foreach($liabilities as $liability){
            $this->drTotal += $liability['dr_amount'];
            $this->crTotal += $liability['cr_amount'];
            
            $result [] = [
                    'particulars' => $liability['ledger_name'],
                    'debit' => $liability['dr_amount'],
                    'credit' => $liability['cr_amount'],
                ]; 
        }
        foreach($assets as $asset){
            $this->drTotal += $asset['dr_amount'];
            $this->crTotal += $asset['cr_amount'];
            
            $result [] = [
                    'particulars' => $asset['ledger_name'],
                    'debit' => $asset['dr_amount'],
                    'credit' => $asset['cr_amount'],
                ]; 
        }
        foreach($expenses as $expense){
            $this->drTotal += $expense['dr_amount'];
            $this->crTotal += $expense['cr_amount'];
            
            $result [] = [
                    'particulars' => $expense['ledger_name'],
                    'debit' => $expense['dr_amount'],
                    'credit' => $expense['cr_amount'],
                ];
        }
        
        foreach($incomes as $income){
            $this->drTotal += $income['dr_amount'];
            $this->crTotal += $income['cr_amount'];
            
            $result [] = [
                    'particulars' => $income['ledger_name'],
                    'debit' => $income['dr_amount'],
                    'credit' => $income['cr_amount'],
                ];
        }
        
        $result[] = ['Particulars' => 'SURPLUS & DEFICIT ACCOUNT', 'debit' => $profit_loss_dr, 'credit' => $profit_loss_cr ];
        
        $total_diff = ($this->drTotal-$this->crTotal);
        if($total_diff > 0){
            $opening_bal_dr = 0;
            $opening_bal_cr = $total_diff;
        }else{
            $opening_bal_dr = abs($total_diff);;
            $opening_bal_cr = 0;
        }
        $this->drTotal += $opening_bal_dr;
        $this->crTotal += $opening_bal_cr;
        
        $result[] = ['Particulars' => 'OPENING BALANCE', 'debit' => $opening_bal_dr, 'credit' => $opening_bal_cr ];
        
        return collect($result);
    }
    
    public function headings(): array
    {
        return [
            'Particulars',
            'Debit',
            'Credit',
        ];
    }
    
    public function title(): string
    {
        return 'Trial Balance';
    }
    
    public function styles(Worksheet $sheet)
    {
        $sheet->getColumnDimension('A')->setWidth(50);
        $sheet->getColumnDimension('B')->setWidth(20);
        $sheet->getColumnDimension('C')->setWidth(20);
            
        $sheet->getStyle('A1:C1')->applyFromArray([
            'font' => [
                'bold' => true,
                'size' => 12,
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00',
                ]
            ]
        ]);
            
        return [];
    }
    
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet;
                    
                // Insert the title row before the column headings (row 1)
                $sheet->insertNewRowBefore(1, 1); // Insert a new row at position 1 for the title
                $sheet->setCellValue('A1', 'Trial Balance From '.date('d-m-Y',strtotime($this->startDate)).' To '.date('d-m-Y',strtotime($this->endDate)));
                $sheet->mergeCells('A1:C1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                $lastRow = $sheet->getHighestRow()+1;
                $sheet->setCellValue('A'.$lastRow, 'TOTAL');
                $sheet->setCellValue('B'.$lastRow, $this->drTotal);
                $sheet->setCellValue('C'.$lastRow, $this->crTotal);
                
                $sheet->getStyle('A2:C'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                    'alignment' => [
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ]
                ]);
                    
                $sheet->getRowDimension(1)->setRowHeight(25);
                $sheet->getRowDimension(2)->setRowHeight(20);
                
                $sheet->getStyle('B3:C'.$lastRow)->applyFromArray([
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ]
                ]);
                
                $sheet->getStyle('A'.$lastRow.':C'.$lastRow)->applyFromArray([
                    'font' => [
                        'bold' => true,
                    ]
                ]);
            }
        ];
    }
}
