<?php

namespace App\Http\Controllers\Accounts;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\AccountSubGroup;
use App\Models\AccountMainGroup;
use App\Models\Accounttype;
use App\Models\Ledger;
use App\Models\LedgerOpenBal;
use App\Models\MenuAccess;
use App\Models\FeeType;
use App\Models\Voucher;

class Ledgers extends Controller
{
    protected $accountId;
    
    public function __construct(){
        $this->accountId = Accounttype::getDefaultAccountId();
    }
    
    public function report(){
        $acgroups = AccountSubGroup::all();
        $add=0;
        if(MenuAccess::have_access(159) == true){ $add=1; }
          
        return view('account.ledger.report',compact('acgroups','add'));
    }
    
    public function getData(Request $request){
        
        $col_order = [null,'ledger_name','subgroup','opening_bal','dc'];
        $total_data = Ledger::whereIn('account_id',[0,$this->accountId])->whereNull('deleted_at')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        
        $query = Ledger::leftJoin('account_sub_group','ledgers.ag_id','=','account_sub_group.acsubgrp_id')->
                    leftJoin('ledger_open_bal','ledgers.ledger_id','=','ledger_open_bal.ledger_id')->
                    whereIn('ledgers.account_id',[0,$this->accountId])->
                    whereNull('ledgers.deleted_at');
            
        if(!empty($request->input('search.value'))){
            $search = $request->input('search.value');
            $query->where(function ($qry) use ($search){
                $qry->where('ledgers.ledger_name','like','%'.$search.'%')->
                    orWhere('account_sub_group.subgroup','like',$search.'%')->
                    orWhere('ledger_open_bal.opening_bal','like','%'.$search.'%')->
                    orWhere('ledgers.dc','like','%'.$search.'%');
                });
            }
        $total_filtered = $query->count();
        $query->select('ledgers.*','account_sub_group.subgroup','ledger_open_bal.opening_bal')->offset($start)->limit($limit);
        if($order && $dir){
            $query->orderBy($order,$dir);
        }else{
            $query->orderBy('ledgers.ledger_id','ASC');
        }
        $ledgers = $query->get();
            
        $edit=$delete=0;
        if(MenuAccess::have_access(160) == true){ $edit=1; }
        if(MenuAccess::have_access(161) == true){ $delete=1; }
        
        $data = array();
        if($ledgers){
            $i=$start;
            foreach($ledgers as $ledger){
                $row['sno'] = ++$i;
                $row['name'] = $ledger->ledger_name;
                $row['group'] = $ledger->subgroup;
                $row['balance'] = $ledger->opening_bal;
                $row['dc'] = $ledger->dc;
                
                $html="";
                if($edit==1 && $ledger->status==1){
                    $html .= '<a href="'.route('accounts.ledger.edit',encrypt($ledger->ledger_id)).'" title="Edit"><i class="icon-sm icon-pencil text-success"></i></a>&nbsp;&nbsp;&nbsp;';
                }
                if($delete==1 && $ledger->status==1){
                    $html .= '<a href="'.route('accounts.ledger.delete',encrypt($ledger->ledger_id)).'" title="Delete" onClick="return confirm(\'Are you sure you want to delete?\');"><i class="icon-sm icon-trash text-danger"></i></a>';
                }
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
    }
    
    public function save(Request $request){
         
        $validated = $request->validate([
                        'ledger_name' => 'required|unique:ledgers,ledger_name',
                        'acsubgrp_id' => 'required',
                    ]);
                    
        $open_bal = $request->opening_balance ?? 0;
        $accountsubgroup = AccountSubGroup::find($request->acsubgrp_id);
        $ledger = Ledger::create([
            'ledger_name' => $request->ledger_name,
            'ag_id' => $request->acsubgrp_id,
            'dc' => $accountsubgroup->dc,
            'account_id' => $this->accountId,
        ]);
        if($ledger){
            $ledgeropenbal = LedgerOpenBal::create([
                                'ledger_id' => $ledger->ledger_id,
                                'opening_bal' => $open_bal,
                                'opening_date' => date('Y-m-d'),
                                'account_id' => $this->accountId,
                            ]);       
                    
            $msg_array = array('message'=>'Ledger Created Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Ledger Could not be Created !','alert-type'=>'error');
        }
        return redirect()->route('accounts.ledgers')->with($msg_array); 
    }
    
    public function edit($lid){
        $acgroups = AccountSubGroup::all();
        $ledger = Ledger::find(decrypt($lid));
          
        return view('account.ledger.edit',compact('acgroups','ledger'));
    }
    
    public function update(Request $request){
        
        $legder_id = decrypt($request->ledger_id);
        $legder = Ledger::find($legder_id);
        
        $validated = $request->validate([
                        'ledger_name' => 'required|unique:ledgers,ledger_name,'.$legder_id.',ledger_id',
                        'acsubgrp_id' => 'required',
                    ]);
                    
        $open_bal = $request->opening_balance ?? 0;
        $accountsubgroup = AccountSubGroup::find($request->acsubgrp_id);
        $update = $legder->update([
            'ledger_name' => $request->ledger_name,
            'ag_id' => $request->acsubgrp_id,
            'dc' => $accountsubgroup->dc,
        ]);
        
        if($update){
            $ledgeropenbal = LedgerOpenBal::updateOrCreate([
                                    'ledger_id' => $legder_id,
                                ],[
                                    'opening_bal' => $open_bal,
                                ]);
                
            $msg_array = array('message'=>'Ledger Updated Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Ledger Could not be Updated !','alert-type'=>'error');
        }
        return redirect()->route('ledger')->with($msg_array);
    }
    
    public function ledger_delete($lid){
        $legder = Ledger::find(decrypt($lid));
        if($legder){
            $ftypes = FeeType::where('ledger_id',decrypt($lid))->count();
            $voucher = Voucher::where('cr_ledger',decrypt($lid))->orWhere('dr_ledger',decrypt($lid))->count();
            
            
            if($ftypes==0 && $voucher==0){
                $legder->delete();
                $msg_array = array('message'=>'Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>' Could not be Deleted !','alert-type'=>'error');
            }
        }
        
        return redirect()->route('ledger')->with($msg_array);
    }
}
