<?php

namespace App\Http\Controllers\Accounts;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Financialyear;
use App\Models\Accounting;
use App\Models\Accounttype;
use App\Models\AccountSubGroup;
use App\Models\Vouchertype;
use App\Models\Voucher;
use App\Models\Ledger;
use App\Models\MenuAccess;

use App\Exports\DaybookExport;
use App\Exports\CashbookExport;
use App\Exports\LedgerExport;
use App\Exports\BalanceSheetExport;
use App\Exports\ProfitLossExport;
use App\Exports\TrialBalanceExport;
use App\Exports\ReceiptPaymentExport;
use App\Exports\AccountGroupExport;

use DB;

use Maatwebsite\Excel\Facades\Excel;

class Reports extends Controller
{
    protected $accountId;
    protected $fyearId;
    protected $acyearId;
    protected $fsDate;
    protected $feDate;

    public function __construct()
    {
        $this->accountId = Accounttype::getDefaultAccountId();
        $this->fyearId = Financialyear::getDefaultFyearId($this->accountId);
        $fyear = Financialyear::find($this->fyearId);
        if ($fyear) {
            $this->fsDate = $fyear->sdate;
            $this->feDate = $fyear->edate;
        } else {
            // Handle missing Financial Year data gracefully
            abort(404, 'Financial Year not found');
        }
    }
    
    public function daybook(Request $request)
    {
        $cur_date = date('Y-m-d');
        if($cur_date > $this->feDate){
            $cur_date = $this->feDate;
        }
        $accountid = $this->accountId;
        
        $fdate = $request->input('fdate') ?? $cur_date;
        $tdate = $request->input('tdate') ?? $cur_date;
        
        $msg_array = [];
        
        $export = MenuAccess::have_access(621) ? 1 : 0;
        
        if(($fdate >= $this->fsDate && $fdate <= $this->feDate) && ($tdate >= $this->fsDate && $tdate <= $this->feDate)){
            if($fdate <= $tdate){
                $vouchers = Voucher::join('vouchertypes','vouchers.vtype_id','=','vouchertypes.id')
                                ->where('vamount','>',0)
                                ->where('account_id',$this->accountId)
                                ->whereBetween('vdate',[$fdate,$tdate])
                                ->select('vouchers.*','vouchertypes.vid')
                                ->get();
                
                return view('account.report.daybook',compact('vouchers','fdate','tdate','export','accountid'));
            }else{
                $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
        }
        
        return redirect()->route('accounts.report.daybook')->with($msg_array);
    }
    
    public function export_daybook(Request $request)
    {
        $validated = $request->validate([
            'fdate' => 'required|date',
            'tdate' => 'required|date|after_or_equal:fdate',
            'accountid' => 'required',
        ]);
        
        $fdate = $request->input('fdate');
        $tdate = $request->input('tdate');
        $accountid = $request->input('accountid');
            
        // Pass the dates to the export class
        return Excel::download(new DaybookExport($fdate, $tdate, $accountid), 'daybook.xlsx');
    }
    
    public function cashbook(Request $request)
    {
        $cur_date = date('Y-m-d');
        if($cur_date > $this->feDate){
            $cur_date = $this->feDate;
        }
        $accountid = $this->accountId;
        
        $fdate = $request->input('fdate') ?? $cur_date;
        $tdate = $request->input('tdate') ?? $cur_date;
        
        $msg_array = [];
        
        $export = MenuAccess::have_access(622) ? 1 : 0;
        
        if(($fdate >= $this->fsDate && $fdate <= $this->feDate) && ($tdate >= $this->fsDate && $tdate <= $this->feDate)){
            if($fdate <= $tdate){
                $pre_date = date('Y-m-d',(strtotime($fdate)-86400));
                
                $pre_open_bal = Accounting::openingBalance($pre_date,$this->accountId,0,1);
                $cur_open_bal = Accounting::openingBalance($tdate,$this->accountId,0,1,$fdate);
                $ledger_open_bal = $pre_open_bal+$cur_open_bal;
                
                $voucher_open_bal = Accounting::voucherBalance(1,$pre_date,$this->accountId,0);
                
                $open_bal = $ledger_open_bal+($voucher_open_bal['dr_amount']-$voucher_open_bal['cr_amount']);
                
                $results = Voucher::whereNull('deleted_at')
                            ->where('vdate', '>=', $fdate)
                            ->where('vdate', '<=', $tdate)
                            ->where('account_id', $this->accountId)
                            ->where(function($query) {
                                $query->where('cr_ledger', 1)
                                    ->orWhere('dr_ledger', 1);
                            })
                            ->get();
                $vouchers = [];
                foreach($results as $result){
                    $dr=$cr=0; $ledger_name='';
                    if($result->dr_ledger==1){
                        $dr = $result->vamount;
                        $ledger_name = 'To '.$result->credit->ledger_name;
                    }else if($result->cr_ledger==1){
                        $cr = $result->vamount;
                        $ledger_name = 'By '.$result->debit->ledger_name;
                    }
                    
                    $vouchers[] = ['ledger_name'=>$ledger_name,'vdate'=>$result->vdate_formatted,'vno'=>$result->vno,'vtype'=>$result->vtype->vtype_name,'dr'=>$dr,'cr'=>$cr];
                }
                
                return view('account.report.cashbook',compact('open_bal','vouchers','fdate','tdate','export','accountid'));
            }else{
                $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
        }
        
        return redirect()->route('accounts.report.cashbook')->with($msg_array);
    }
    
    public function export_cashbook(Request $request)
    {
        $validated = $request->validate([
            'fdate' => 'required|date',
            'tdate' => 'required|date|after_or_equal:fdate',
            'accountid' => 'required',
        ]);
        
        $fdate = $request->input('fdate');
        $tdate = $request->input('tdate');
        $accountid = $request->input('accountid');
            
        // Pass the dates to the export class 
        return Excel::download(new CashbookExport($fdate, $tdate, $accountid), 'cashbook.xlsx');
    }
    
    public function ledger(Request $request)
    {
        $cur_date = date('Y-m-d');
        if($cur_date > $this->feDate){
            $cur_date = $this->feDate;
        }
        $accountid = $this->accountId;
        
        $fdate = $request->input('fdate') ?? $cur_date;
        $tdate = $request->input('tdate') ?? $cur_date;
        $ledgerid = $request->input('ledgerid');
        
        $msg_array = [];
        
        $export = MenuAccess::have_access(626) ? 1 : 0;
        
        if(($fdate >= $this->fsDate && $fdate <= $this->feDate) && ($tdate >= $this->fsDate && $tdate <= $this->feDate)){
            if($fdate <= $tdate){
                $ledgers = Ledger::all();
                $vouchers = []; $open_bal=0;
                $ledgerdet = [];
                if($ledgerid!=null){
                    
                    $pre_date = date('Y-m-d',(strtotime($fdate)-86400));
                    
                    $ledgerdet = Ledger::find($ledgerid);
                    $pre_open_bal = Accounting::openingBalance($pre_date,$this->accountId,0,$ledgerid);
                    $cur_open_bal = Accounting::openingBalance($tdate,$this->accountId,0,$ledgerid,$fdate);
                    $ledger_open_bal = $pre_open_bal+$cur_open_bal;
                    
                    if(in_array($ledgerdet->sub->acmaingrp_id, [3, 4, 5, 6, 10, 11])){
                        $voucher_total = Accounting::ledgerTotal($ledgerid,$pre_date,$this->accountId,$this->fsDate);
                    }else{
                        $voucher_total = Accounting::ledgerTotal($ledgerid,$pre_date,$this->accountId);
                    }
                    
                    if($ledgerdet->dc=='dr'){
                        $voucher_open_bal = $voucher_total['dr_amount']-$voucher_total['cr_amount'];
                    }else{
                        $voucher_open_bal = $voucher_total['cr_amount']-$voucher_total['dr_amount'];
                    }
                    
                    $open_bal = $ledger_open_bal+$voucher_open_bal;
                    
                    $results = Voucher::whereNull('deleted_at')
                                ->where('vdate', '>=', $fdate)
                                ->where('vdate', '<=', $tdate)
                                ->where('account_id', $this->accountId)
                                ->where(function($query) use ($ledgerid) {
                                    $query->where('cr_ledger', $ledgerid)
                                        ->orWhere('dr_ledger', $ledgerid);
                                })
                                ->get();
                    
                    foreach($results as $result){
                        $dr=$cr=0; $ledger_name='';
                        if($result->dr_ledger==$ledgerid){
                            $dr = $result->vamount;
                            $ledger_name = 'To '.$result->credit->ledger_name;
                        }else if($result->cr_ledger==$ledgerid){
                            $cr = $result->vamount;
                            $ledger_name = 'By '.$result->debit->ledger_name;
                        }
                        
                        $vouchers[] = ['ledger_name'=>$ledger_name,'vdate'=>$result->vdate_formatted,'vno'=>$result->vno,'vtype'=>$result->vtype->vtype_name,'dr'=>$dr,'cr'=>$cr];
                    }
                }
                
                return view('account.report.ledger',compact('open_bal','vouchers','ledgers','ledgerdet','ledgerid','fdate','tdate','export','accountid'));
            }else{
                $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
        }
        
        return redirect()->route('accounts.report.ledger')->with($msg_array);
    }
    
    public function export_ledger(Request $request)
    {
        $validated = $request->validate([
            'ledgerid' => 'required',
            'fdate' => 'required|date',
            'tdate' => 'required|date|after_or_equal:fdate',
            'accountid' => 'required',
        ]);
        
        $ledgerid = $request->input('ledgerid');
        $fdate = $request->input('fdate');
        $tdate = $request->input('tdate');
        $accountid = $request->input('accountid');
            
        // Pass the dates to the export class 
        return Excel::download(new LedgerExport($ledgerid, $fdate, $tdate, $accountid), 'ledgerbook.xlsx');
    }
    
    public function accountgroup(Request $request)
    {
        $cur_date = date('Y-m-d');
        if($cur_date > $this->feDate){
            $cur_date = $this->feDate;
        }
        $accountid = $this->accountId;
        
        $fdate = $request->input('fdate') ?? $cur_date;
        $tdate = $request->input('tdate') ?? $cur_date;
        $agid = $request->input('agid') ?? null;
        
        $msg_array = [];
        
        $subgroups = AccountSubGroup::all();
        
        $export = MenuAccess::have_access(645) ? 1 : 0;
        
        if(($fdate >= $this->fsDate && $fdate <= $this->feDate) && ($tdate >= $this->fsDate && $tdate <= $this->feDate)){
            if($fdate <= $tdate){
                $vouchers = [];
                if($agid){
                    $ledgers = Ledger::where('ag_id',$agid)->get();
                    foreach($ledgers as $ledger){
                        $dr_amount=$cr_amount=0;
                        $data = Accounting::voucherBalance($ledger->ledger_id,$tdate,$this->accountId,0,$fdate);
                        if($data['dr_amount'] > 0 || $data['cr_amount'] > 0)
                        {
                            if($data['dr_amount'] > $data['cr_amount']){
                                $dr_amount = $data['dr_amount']-$data['cr_amount'];
                            }
                            if($data['dr_amount'] < $data['cr_amount']){
                                $cr_amount = $data['cr_amount']-$data['dr_amount'];
                            }
                            
                            $vouchers[] = array('ledger_name' => $ledger->ledger_name,'dr_amount' => $dr_amount,'cr_amount' => $cr_amount);
                        }
                    }
                }
                return view('account.report.accountgroup',compact('vouchers','subgroups','agid','fdate','tdate','export','accountid'));
            }else{
                $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
        }
        
        return redirect()->route('accounts.report.accountgroup')->with($msg_array);
    }
    
    public function export_accountgroup(Request $request){
        $validated = $request->validate([
            'fdate' => 'required|date',
            'tdate' => 'required|date|after_or_equal:fdate',
            'accountid' => 'required',
            'agid' => 'required',
        ]);
        
        $fdate = $request->input('fdate');
        $tdate = $request->input('tdate');
        $agid = $request->input('agid');
        $accountid = $request->input('accountid');
            
        return Excel::download(new AccountGroupExport($agid, $fdate, $tdate, $accountid), 'accountgroup.xlsx');
    }
    
    public function balance_sheet(Request $request){
        $cur_date = date('Y-m-d');
        if($cur_date > $this->feDate){
            $cur_date = $this->feDate;
        }
        $accountid = $this->accountId;
        
        $fdate = $request->input('fdate') ?? $cur_date;
        $tdate = $request->input('tdate') ?? $cur_date;
        
        $msg_array = [];
        
        $export = MenuAccess::have_access(636) ? 1 : 0;
        
        if(($fdate >= $this->fsDate && $fdate <= $this->feDate) && ($tdate >= $this->fsDate && $tdate <= $this->feDate)){
            if($fdate <= $tdate){
                $open_bal = Accounting::openingBalance($tdate,$this->accountId);
                    
                $liab_count = Accounting::getLedgers("liability")->count();
                $liabilities = Accounting::voucherAmount("liability",$accountid,$tdate,0,$liab_count);
                    
                $asset_count = Accounting::getLedgers("asset")->count();
                $assets = Accounting::voucherAmount("asset",$accountid,$tdate,0,$asset_count);
                    
                $expense_count = Accounting::getLedgers("expense")->count();
                $expenses = Accounting::profitLoss("expense",$accountid,$tdate,'',0,$expense_count,0,$this->fsDate);
                    
                $income_count = Accounting::getLedgers("income")->count();
                $incomes = Accounting::profitLoss("income",$accountid,$tdate,'',0,$income_count,0,$this->fsDate);
                  
                return view('account.report.balancesheet',compact('fdate','tdate','export','accountid','open_bal','liabilities','assets','expenses','incomes'));
            }else{
                $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
        }
        
        return redirect()->route('accounts.report.balancesheet')->with($msg_array);
    }
    
    public function export_balance_sheet(Request $request){
        $validated = $request->validate([
            'fdate' => 'required|date',
            'tdate' => 'required|date|after_or_equal:fdate',
            'accountid' => 'required',
        ]);
        
        $fdate = $request->input('fdate');
        $tdate = $request->input('tdate');
        $accountid = $request->input('accountid');
            
        return Excel::download(new BalanceSheetExport($fdate, $tdate, $accountid), 'balancesheet.xlsx');
    }
    
    public function profit_loss(Request $request){
        $cur_date = date('Y-m-d');
        if($cur_date > $this->feDate){
            $cur_date = $this->feDate;
        }
        $accountid = $this->accountId;
        
        $fdate = $request->input('fdate') ?? $cur_date;
        $tdate = $request->input('tdate') ?? $cur_date;
        
        $msg_array = [];
        
        $export = MenuAccess::have_access(640) ? 1 : 0;
        
        if(($fdate >= $this->fsDate && $fdate <= $this->feDate) && ($tdate >= $this->fsDate && $tdate <= $this->feDate)){
            if($fdate <= $tdate){
                $open_bal = Accounting::openingBalance($tdate,$this->accountId);
                    
                $expense_count = Accounting::getLedgers("expense")->count();
                $expenses = Accounting::profitLoss("expense",$accountid,$fdate,$tdate,0,$expense_count,0,$this->fsDate);
                    
                $income_count = Accounting::getLedgers("income")->count();
                $incomes = Accounting::profitLoss("income",$accountid,$fdate,$tdate,0,$income_count,0,$this->fsDate); 
                  
                return view('account.report.profitloss',compact('fdate','tdate','export','accountid','expenses','incomes'));
            }else{
                $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
        }
        
        return redirect()->route('accounts.report.profitloss')->with($msg_array);
    }
    
    public function export_profit_loss(Request $request){
        $validated = $request->validate([
            'fdate' => 'required|date',
            'tdate' => 'required|date|after_or_equal:fdate',
            'accountid' => 'required',
        ]);
        
        $fdate = $request->input('fdate');
        $tdate = $request->input('tdate');
        $accountid = $request->input('accountid');
            
        return Excel::download(new ProfitLossExport($fdate, $tdate, $accountid), 'profitloss.xlsx');
    }
    
    public function trial(Request $request){
        $cur_date = date('Y-m-d');
        if($cur_date > $this->feDate){
            $cur_date = $this->feDate;
        }
        $accountid = $this->accountId;
        
        $fdate = $request->input('fdate') ?? $cur_date;
        $tdate = $request->input('tdate') ?? $cur_date;
        
        $msg_array = [];
        
        $export = MenuAccess::have_access(641) ? 1 : 0;
        
        if(($fdate >= $this->fsDate && $fdate <= $this->feDate) && ($tdate >= $this->fsDate && $tdate <= $this->feDate)){
            if($fdate <= $tdate){
                $liab_count = Accounting::getLedgers("liability")->count();
                $liabilities = Accounting::voucherAmount("liability",$accountid,$tdate,0,$liab_count);
                    
                $asset_count = Accounting::getLedgers("asset")->count();
                $assets = Accounting::voucherAmount("asset",$accountid,$tdate,0,$asset_count);
                    
                $expense_count = Accounting::getLedgers("expense")->count();
                $expenses = Accounting::profitLoss("expense",$accountid,$tdate,'',0,$expense_count,0,$this->fsDate);
                    
                $income_count = Accounting::getLedgers("income")->count();
                $incomes = Accounting::profitLoss("income",$accountid,$tdate,'',0,$income_count,0,$this->fsDate);
                
                $profit_loss_cr=$profit_loss_dr=0;
                $profit_loss = DB::table('ledger_total_bal')->sum('ledger_amount');
                if($profit_loss > 0){
                    $profit_loss_cr = $profit_loss;
                }else{
                    $profit_loss_dr = abs($profit_loss);
                }
                
                return view('account.report.trial',compact('fdate','tdate','export','accountid','liabilities','assets','expenses','incomes','profit_loss_cr','profit_loss_dr'));
            }else{
                $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
        }
        
        return redirect()->route('accounts.report.trial')->with($msg_array);
    }
    
    public function export_trial(Request $request){
        $validated = $request->validate([
            'fdate' => 'required|date',
            'tdate' => 'required|date|after_or_equal:fdate',
            'accountid' => 'required',
        ]);
        
        $fdate = $request->input('fdate');
        $tdate = $request->input('tdate');
        $accountid = $request->input('accountid');
            
        return Excel::download(new TrialBalanceExport($fdate, $tdate, $accountid), 'trialbalance.xlsx');
    }
    
    public function receipt_payment(Request $request){
        $cur_date = date('Y-m-d');
        if($cur_date > $this->feDate){
            $cur_date = $this->feDate;
        }
        $accountid = $this->accountId;
        
        $fdate = $request->input('fdate') ?? $cur_date;
        $tdate = $request->input('tdate') ?? $cur_date;
        
        $msg_array = [];
        
        $export = MenuAccess::have_access(642) ? 1 : 0;
        
        if(($fdate >= $this->fsDate && $fdate <= $this->feDate) && ($tdate >= $this->fsDate && $tdate <= $this->feDate)){
            if($fdate <= $tdate){
                $openings = Accounting::cbOpening($fdate,$this->accountId);
                $ledger_count = Accounting::getLedgers("all")->count();
                $voucher_amounts = Accounting::rpAccount($fdate,$tdate,$this->accountId,0,$ledger_count);
                $closings = Accounting::cbClosing($tdate,$this->accountId);
                
                return view('account.report.receiptpayment',compact('fdate','tdate','export','accountid','openings','voucher_amounts','closings'));
            }else{
                $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Invalid Date Range !','alert-type'=>'error');
        }
        
        return redirect()->route('accounts.report.receiptpayment')->with($msg_array);
    }
        
    public function export_receipt_payment(Request $request){
        $validated = $request->validate([
            'fdate' => 'required|date',
            'tdate' => 'required|date|after_or_equal:fdate',
            'accountid' => 'required',
        ]);
        
        $fdate = $request->input('fdate');
        $tdate = $request->input('tdate');
        $accountid = $request->input('accountid');
            
        return Excel::download(new ReceiptPaymentExport($fdate, $tdate, $accountid), 'receipt_payment.xlsx');
    }
}