<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use App\Models\Acyear;
use App\Models\Course;
use App\Models\Semester;
use App\Models\FeeCategory;
use App\Models\FeeType;
use App\Models\FeeTerm;
use App\Models\FeeFine;
use App\Models\FeeStructure;
use App\Models\FeeStructureType;
use App\Models\Paymentmode;
use App\Models\Ledger;

class FeeSettings extends Controller
{
    public function category(){
        
        $categories = FeeCategory::all();
        
        return view('feesettings.category',compact('categories'));
    }
    
    public function category_save(Request $request){
        
        $validated = $request->validate([
            'category_name' => 'required|unique:fee_categories,category_name',
        ]);
        
        $category = FeeCategory::create([
            'category_name' => $request->category_name,
        ]);
        
        if($category){
            $msg_array = array('message'=>'Fee Category Added Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Fee Category Could not be Added !','alert-type'=>'error');
        }
        return redirect()->route('feecategory')->with($msg_array);
    }
    
    public function category_delete($cid){
        
        $category = FeeCategory::find(decrypt($cid));
        
        if($category){
            if($category->count() > 0){
                $category->delete();
                $msg_array = array('message'=>'Fee Category Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Fee Category Could not be Deleted !','alert-type'=>'error');
            }
        }
        
        return redirect()->route('feecategory')->with($msg_array);
    }
    
    public function type(){
        
        $types = FeeType::all();
        
        return view('feesettings.type',compact('types'));
    }
    
    public function type_save(Request $request){
        
        $validated = $request->validate([
            'type_name' => 'required|unique:fee_types,type_name',
        ]);
        
        $ledger = Ledger::create([
                        'ledger_name' => $request->type_name,
                        'ag_id' => 9,
                        'dc' => 'cr',
                        'account_id' => 0
                    ]);
        if($ledger){
            $type = FeeType::create([
                'type_name' => $request->type_name,
                'ledger_id' => $ledger->ledger_id
            ]);
            
            if($type){
                $msg_array = array('message'=>'Fee Type Added Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Fee Type Could not be Added !','alert-type'=>'error');
            }
        }else{
            $msg_array = array('message'=>'Fee Type Could not be Added !','alert-type'=>'error');
        }
            
        return redirect()->route('feetype')->with($msg_array);
    }
    
    public function type_delete($tid){
        
        $type = FeeType::find(decrypt($tid));
        
        if($type){
            if($type->count() > 0){
                $type->delete();
                $msg_array = array('message'=>'Fee Type Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Fee Type Could not be Deleted !','alert-type'=>'error');
            }
        }
        
        return redirect()->route('feetype')->with($msg_array);
    }
    
    public function term(){
        
        $terms = FeeTerm::all();
        
        return view('feesettings.term',compact('terms'));
    }
    
    public function term_save(Request $request){
        
        $validated = $request->validate([
            'term_name' => 'required|unique:fee_terms,term_name',
        ]);
        
        $term = FeeTerm::create([
            'term_name' => $request->term_name,
        ]);
        
        if($term){
            $msg_array = array('message'=>'Fee Term Added Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Fee Term Could not be Added !','alert-type'=>'error');
        }
        return redirect()->route('feeterm')->with($msg_array);
    }
    
    public function term_delete($tid){
        
        $term = FeeTerm::find(decrypt($tid));
        if($term){
            if($term->count() > 0){
                $term->delete();
                $msg_array = array('message'=>'Fee Term Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Fee Term Could not be Deleted !','alert-type'=>'error');
            }
        }
        
        return redirect()->route('feeterm')->with($msg_array);
    }
    
    public function fine(){
        
        $acyears = Acyear::all();
        $courses = Course::all();
        $feetypes = FeeType::all();
        
        return view('feesettings.fine',compact('acyears','courses','feetypes'));
    }
    
    public function fine_get(Request $request){
        
        $fromdays[''] = $todays[''] = 'Select'; $todays[-1] = 'To End';
        $ac_det = Acyear::find($request->aid);
        $course_det = Course::find($request->cid);
        $type_det = FeeType::find($request->tid);
        $fine_type = $request->fid;
        
        $fines = FeeFine::where(array('ac_id'=>$request->aid,'course_id'=>$request->cid,'type_id'=>$request->tid,'fine_type'=>$fine_type))->get();
        if($fine_type==0){
            for($i=1;$i<=31;$i++){
                $fromdays[$i] = $i;
                $todays[$i] = $i;
            }
        }else if($fine_type==1){
            for($i=1;$i<=12;$i++){
                $fromdays[$i] = $i;
                $todays[$i] = $i;
            }
        }
        return view('feesettings.fine_get',compact('ac_det','course_det','type_det','fine_type','fines','fromdays','todays'));
    }
    
    public function fine_save(Request $request){
        
        $validated = $request->validate([
            'from' => 'required',
            'from.*' => 'required',
            'to' => 'required',
            'to.*' => 'required',
            'amount' => 'required',
            'amount.*' => 'required',
        ]);
        
        $from = $request->from;
        $to = $request->to;
        $amount = $request->amount;
        $in_count=0;
        
        foreach($from as $key=>$value){
            $in_from = $value;
            $in_to = $to[$key];
            $in_amount = $amount[$key];
            
            $fine = FeeFine::create([
                'ac_id' => $request->ac_id,
                'course_id' => $request->course_id,
                'type_id' => $request->type_id,
                'fine_type' => $request->fine_type,
                'from' => $in_from,
                'to' => $in_to,
                'amount' => $in_amount,
            ]);
            if($fine){ $in_count++; }
        }
        
        if($in_count > 0){
            $msg_array = array('message'=>'Fine Saved Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Fine Could not be Saved !','alert-type'=>'error');
        }
        
        return redirect()->route('fine')->with($msg_array);
    }
    
    public function fine_row($fine_type){
        
        $fromdays[''] = $todays[''] = 'Select'; $todays[-1] = 'To End';
        if($fine_type==0){
            for($i=1;$i<=31;$i++){
                $fromdays[$i] = $i;
                $todays[$i] = $i;
            }
        }else if($fine_type==1){
            for($i=1;$i<=12;$i++){
                $fromdays[$i] = $i;
                $todays[$i] = $i;
            }
        }
        
        return view('feesettings.fine_row',compact('fromdays','todays'));
    }
    
    public function structure(){
        
        $acyears = Acyear::all();
        $courses = Course::all();
        $categories = FeeCategory::all();
        $terms = FeeTerm::all();
        
        return view('feesettings.structure',compact('acyears','courses','categories','terms'));
    }
    
    public function structure_get(Request $request){
        
        $ac_det = Acyear::find($request->aid);
        $crs_det = Course::find($request->cid);
        $sem_det = Semester::find($request->sid);
        $cat_det = FeeCategory::find($request->ctid);
        $term_det = FeeTerm::find($request->tid);
        $feetypes = FeeType::all();
        
        $stypes = FeeStructureType::Join('fee_structures','fee_structure_types.structure_id','=','fee_structures.structure_id')->where(array('fee_structures.ac_id'=>$request->aid,'fee_structures.course_id'=>$request->cid,'fee_structures.semester_id'=>$request->sid,'fee_structures.category_id'=>$request->ctid,'fee_structures.term_id'=>$request->tid))->select('fee_structure_types.*')->latest()->get();

        return view('feesettings.structure_get',compact('ac_det','crs_det','sem_det','cat_det','term_det','feetypes','stypes'));
    }
    
    public function structure_save(Request $request){
        
        $validated = $request->validate([
            'ac_id' => 'required',
            'course_id' => 'required',
            'semester_id' => 'required',
            'category_id' => 'required',
            'term_id' => 'required',
        ]);
        
        $type = $request->type;
        $amount = $request->amount;
        $start = $request->start;
        $end = $request->end;
        $in_count=0;
        
        $feestructure = FeeStructure::firstOrCreate([
            'ac_id' => $request->ac_id,
            'course_id' => $request->course_id,
            'semester_id' => $request->semester_id,
            'category_id' => $request->category_id,
            'term_id' => $request->term_id
        ]);
           
        if($feestructure){
            foreach($type as $key=>$value){
                if($amount[$key] > 0 && $start[$key]!='0000-00-00' && $start[$key]!='1970-01-01' && $end[$key]!='0000-00-00' && $end[$key]!='1970-01-01'){
                    $fstype = FeeStructureType::updateOrCreate([
                        'structure_id' => $feestructure->structure_id,
                        'type_id' => $value],[
                        'amount' => $amount[$key],
                        'start_date' => $start[$key],
                        'end_date' => $end[$key]
                    ]);
                    
                    if($fstype){ $in_count++; }
                }
            }
        }
        
        if($in_count > 0){
            $msg_array = array('message'=>'Fee Structure Saved Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Fee Structure Could not be Saved !','alert-type'=>'error');
        }
        
        return redirect()->route('structure')->with($msg_array);
    }
    
    public function paymode(){
        
        $paymodes = Paymentmode::where('status',1)->get();
        
        return view('feesettings.paymode',compact('paymodes'));
    }
    
    public function paymode_save(Request $request){
        
        $validated = $request->validate([
            'mode_name' => 'required|unique:paymentmodes,mode_name',
        ]);
        
        $paymode = Paymentmode::create([
            'mode_name' => $request->mode_name,
        ]);
        
        if($paymode){
            $msg_array = array('message'=>'Payment Mode Added Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Payment Mode Could not be Added !','alert-type'=>'error');
        }
        return redirect()->route('paymode')->with($msg_array);
    }
    
    public function paymode_delete($mid){
        
        $paymode = Paymentmode::find(decrypt($mid));
        
        if($paymode){
            if($paymode->count() > 0){
                $paymode->update([
                    'status' => 0
                    ]);
                $msg_array = array('message'=>'Payment Mode Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Payment Mode Could not be Deleted !','alert-type'=>'error');
            }
        }
        
        return redirect()->route('paymode')->with($msg_array);
    }
}
