<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use App\Models\Acyear;
use App\Models\Course;
use App\Models\Semester;
use App\Models\Student;
use App\Models\Paymentmode;
use App\Models\Fee;
use App\Models\User;
use App\Models\MenuAccess;
use App\Models\FeetypeAmount;
use App\Models\Otherfee;
use App\Models\FeeType;
use App\Models\Promote;
use App\Models\ScholarshipStudent;
use App\Models\FeeStructure;
use App\Models\FeeStructureType;
use App\Models\StudentFeeStuType;
use App\Models\StudentFeeStru;
use App\Models\FeeCategory;
use App\Models\FeeTerm;
use App\Models\Voucher;
use App\Models\Financialyear;
use App\Models\Accounttype;

use PDF;

use Illuminate\Support\Facades\DB;

class Fees extends Controller
{
    protected $accountId;
    protected $fyearId;
    protected $acyearId;
    protected $fsDate;
    protected $feDate;

    public function __construct()
    {
        $this->accountId = Accounttype::getDefaultAccountId();
        $this->fyearId = Financialyear::getDefaultFyearId($this->accountId);
        $fyear = Financialyear::find($this->fyearId);
        if ($fyear) {
            $this->fsDate = $fyear->sdate;
            $this->feDate = $fyear->edate;
        } else {
            // Handle missing Financial Year data gracefully
            abort(404, 'Financial Year not found');
        }
        
        $acyear = Acyear::where('default1',1)->first();
        if ($acyear) {
            $this->acyearId = $acyear->ac_id;
            $this->sDate = $acyear->sdate;
            $this->eDate = $acyear->edate;
        } else {
            // Handle missing Academic Year data gracefully
            abort(404, 'Academic Year not found');
        }
    }
    
    public function payment(){
        
        $students = Student::where('tcstatus', '0')->get();
        $paymodes = Paymentmode::where('status', 1)->get();
        
        return view('fees.payment',compact('students','paymodes'));
    }
    
    public function get_payment(Request $request){
            
        $validated = $request->validate([
            'student_id' => 'required|exists:students,student_id',
            'paid_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'mode_id' => 'required|exists:paymentmodes,mode_id',
        ]);
            
        $fees = [];
        $student_id = $request->student_id;
        $paid_date = $request->paid_date;
        $mode_id = $request->mode_id;
            
        $students = Student::where('tcstatus', '0')->get();
        $paymodes = Paymentmode::where('status', 1)->get();
            
        try {
          
            $student_det = Student::findOrFail($student_id);
            $cur_acyear = Acyear::where('default1', 1)->whereNull('deleted_at')->first();
            $ac_id = $cur_acyear->ac_id;
                
            $paid_history = Fee::where('student_id', $student_id)->get();
            $grouped_payments = $paid_history->groupBy('receipt_no');
            $next_rec_no = Fee::max('receipt_no') + 1;
                
            $fee_category = $student_det->category_id;
            if($fee_category > 0){
                if($paid_date <= date('Y-m-d')){
                    $fees = Fee::studentFees($student_id,$paid_date);
                }
            }
                
            $semester = Student::join('promotes', 'students.student_id', '=', 'promotes.student_id')
                ->join('courses', 'promotes.course_id', '=', 'courses.course_id')
                ->join('semesters', 'promotes.semester_id', '=', 'semesters.semester_id')
                ->where('promotes.ac_id', $ac_id)
                ->where('promotes.student_id', $student_id)
                ->first();
                
            $scholarshipstudent = ScholarshipStudent::join('scholarship', 'scholarshipstudent.sc_id', '=', 'scholarship.sc_id')
                ->where('scholarshipstudent.student_id', $student_id)
                ->get();
               $crs=Student::join('promotes','students.student_id','=','promotes.student_id')
->join('courses','courses.course_id','=','promotes.course_id')
->where('promotes.ac_id',$ac_id)->where('promotes.student_id',$student_id)
->get();

            return view('fees.get_payment',compact('crs','students','student_id','paid_date','paymodes','mode_id','fees','student_det','next_rec_no','grouped_payments','semester'));
        } catch (\Exception $e) {
            // Log the error message for debugging
            \Log::error('Error fetching payment details: ' . $e->getMessage());
                
            // Provide a user-friendly error message
            return redirect()->route('fee')->with('error', 'An error occurred while fetching payment details.');
        }
    }
        
    public function save_payment(Request $request){
        $validated = $request->validate([
            'posted' => 'required|array',
            'sid' => 'required|string',
            'pdate' =>  "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'mode' => 'required|exists:paymentmodes,mode_id', // Validate payment mode
        ]);
            
        $feetypeData = Feetype::pluck('ledger_id', 'type_id')->toArray();
            
        DB::beginTransaction();
            
        try {
            $rec_no = Fee::max('receipt_no') + 1;
            $fgroup_id = Fee::withTrashed()->max('group_id') + 1 ?? 1;
            $data = [
                'group_id' => $fgroup_id,
                'receipt_no' => $rec_no,
                'student_id' => decrypt($validated['sid']),
                'paid_date' => $validated['pdate'],
                'mode_id' => $validated['mode'],
                'account_id' => $this->accountId,
                'created_by' => auth()->user()->user_id,
            ];
                
            $i = $total_discount = 0; $vouchers = [];
            foreach ($validated['posted'] as $ids) {
                $fee_det = explode('-', $ids);
                $type_id = $fee_det[3];
                $discount = $request->input("discount".$ids);
                
                $data['ac_id'] = $fee_det[0];
                $data['semester_id'] = $fee_det[1];
                $data['term_id'] = $fee_det[2];
                $data['type_id'] = $type_id;
                $data['fee_amount'] = $request->input("fee_amount".$ids);
                $data['fine_amount'] = $request->input("fine_amount".$ids);
                $data['discount'] = $discount;
                $data['remarks'] = $request->input("remarks");
                    
                // Create the fee record in the database
                Fee::create($data);
                    
                $ledger_id = $feetypeData[$type_id] ?? null;
                if($ledger_id){
                    $total_amount = $data['fee_amount'] + $data['fine_amount'];
                    if (isset($vouchers[$ledger_id])) {
                        $vouchers[$ledger_id] += $total_amount;
                    }else{
                        $vouchers[$ledger_id] = $total_amount;
                    }
                }
                    
                if($discount!='' && $discount>0){
                    $total_discount += $discount;
                }
                
                $i++;
            }
                
            if($i>0){
                if(!empty($vouchers)){
                    $group_id = Voucher::withTrashed()->max('group_id') + 1 ?? 1;
                    $vno = Voucher::where('vtype_id', 2)
                          ->where('account_id', $this->accountId)
                          ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                          ->max('vno') + 1 ?? 1;
                        
                    $vdata = [
                        'group_id' => $group_id,
                        'vtype_id' => 2,
                        'vno' => $vno,
                        'vdate' => $data['paid_date'],
                        'dr_ledger' => $data['mode_id'],
                        'table_name' => 'fees',
                        'table_group_id' => $fgroup_id,
                        'account_id' => $this->accountId,
                        'created_by' => auth()->user()->user_id
                    ];
                    
                    $k=1;
                    foreach($vouchers as $key => $value){
                        $vdata['vorder'] = $k;
                        $vdata['cr_ledger'] = $key;
                        $vdata['vamount'] = $value;
                        
                        $vsave = Voucher::create($vdata);
                        if($vsave){
                            $k++;
                        }
                    }
                }
                
                if($total_discount>0){
                    $dgroup_id = Voucher::withTrashed()->max('group_id') + 1 ?? 1;
                    $dvno = Voucher::where('vtype_id', 1)
                          ->where('account_id', $this->accountId)
                          ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                          ->max('vno') + 1 ?? 1;
                        
                    $dvdata = [
                        'group_id' => $dgroup_id,
                        'vtype_id' => 1,
                        'vno' => $dvno,
                        'vorder' => 1,
                        'vdate' => $data['paid_date'],
                        'dr_ledger' => 8,
                        'cr_ledger' => $data['mode_id'],
                        'vamount' => $total_discount,
                        'table_name' => 'fees',
                        'table_group_id' => $fgroup_id,
                        'account_id' => $this->accountId,
                        'created_by' => auth()->user()->user_id
                    ];
                    
                    Voucher::create($dvdata);
                }
            }
            
            DB::commit();
            
            echo "<script>window.open('".route('fee.print', encrypt($rec_no))."');</script>";
            return redirect()->route('fee')->with('success', 'Fee Paid Successfully!');
        } catch (\Exception $e) {
            DB::rollBack();
                
            \Log::error('Error saving payment: ' . $e->getMessage());
                
            return redirect()->route('fee')->with('error', 'An error occurred while processing the payment.');
        }
    }
        
    public function report(){
        
        $paymodes = Paymentmode::where('status',1)->get();
        $staffs = User::all();
        $courses = Course::all();
        
        return view('fees.report',compact('courses','paymodes','staffs'));
    }
    
    public function get_report(Request $request){
        
        $validated = $request->validate([
                'fdate' => 'required|date',
                'tdate' => 'required|date',
            ]);
        $fees=array();
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        
        $ac_id=$cur_acyear->ac_id;
        $fdate = $request->fdate;
        $tdate = $request->tdate;
        $course_id = $request->course;
        $semester_id = $request->semester;
        $mode_id = $request->mode;
        $staff_id = $request->staff;
        $courses = Course::all();
        $semesters = array();
        if($course_id!=''){
            $semesters = Semester::where('course_id',$course_id)->get();
        }
        
        $paymodes = Paymentmode::where('status',1)->get();
        $staffs = User::all();
        
        $query = Fee::join('students','fees.student_id','=','students.student_id')->
                      join('promotes','students.student_id','=','promotes.student_id')->
                    where('fees.status',1)->
                    where('fees.paid_date','>=',$fdate)->
                    where('fees.paid_date','<=',$tdate);
                    if($course_id!=''){ $query->where('students.course_id',$course_id); }
                    if($semester_id!=''){ $query->where('promotes.semester_id',$semester_id); }
                    if($mode_id!=''){ $query->where('fees.mode_id',$mode_id); }
                    if($staff_id!=''){ $query->where('fees.created_by',$staff_id); }

        $results = $query->select('fees.receipt_no','fees.paid_date')->groupBy(['fees.receipt_no','fees.paid_date'])->get();
   
        foreach($results as $result){
            $fee_det = Fee::join('students','fees.student_id','=','students.student_id')->
                        join('paymentmodes','fees.mode_id','=','paymentmodes.mode_id')->
                        join('users','fees.created_by','=','users.user_id')->
                        where('fees.status',1)->
                        where('fees.receipt_no',$result->receipt_no)->
                        where('fees.paid_date',$result->paid_date)->
                        select('fees.fee_id','fees.receipt_no','fees.paid_date','students.adno','students.name',DB::raw('sum(fee_amount) as fee'),DB::raw('sum(fine_amount) as fine'),DB::raw('sum(discount) as discount'),'paymentmodes.mode_name','users.name as uname')->get()->first();
            $fees[] = array('fee_id'=>$fee_det->fee_id,'recno'=>$fee_det->receipt_no,'pdate'=>$fee_det->paid_date,'adno'=>$fee_det->adno,'name'=>$fee_det->name,'fee'=>$fee_det->fee,'fine'=>$fee_det->fine,'mode'=>$fee_det->mode_name,'staff'=>$fee_det->uname,'discount'=>$fee_det->discount);
        }
        
        return view('fees.get_report',compact('courses','course_id','semesters','semester_id','paymodes','mode_id','staffs','staff_id','fdate','tdate','fees'));
    }
    
    public function delete($fid){
        
        $fee = Fee::find(decrypt($fid));
        $groupid = $fee->group_id;
        $studid = $fee->student_id;
        $created = $fee->created_by;
        if($fee){
            $delete = Fee::where('group_id',$groupid)->where('student_id',$studid)->where('created_by',$created)->delete();
            if($delete){
                Voucher::where('table_name','fees')->where('table_group_id',$groupid)->where('created_by',$created)->delete();
                
                $msg_array = array('message'=>'Receipt Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Receipt Could not be Deleted !','alert-type'=>'error');
            }
        }
        
        return redirect()->route('freport')->with($msg_array);
    }
    
    public function print_bill($recno){
        
        $ins_det = instituteDetails();
        $fee_det = Fee::where('receipt_no',decrypt($recno))->first();
        $fee_item_det = Fee::where('receipt_no',decrypt($recno))->get();
         
        return view('fees.print_bill',compact('fee_det','fee_item_det','ins_det'));
    }
    
    public function due_report(){
        
        $courses = Course::all();
        
        return view('fees.due_report',compact('courses'));
    }
    
    public function due_ajax(Request $request){
        
        $col_order = ['adno','name','course_name','semester_name','',''];
        $total_data = Student::whereNull('deleted_at')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        
        $course = $request->get('course');
        $semester = $request->get('semester');
        $cdate = $request->get('cdate');
        
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        
        $query  =   Student::join('promotes','students.student_id','=','promotes.student_id')->
                        join('courses','promotes.course_id','=','courses.course_id')->
                        join('semesters','promotes.semester_id','=','semesters.semester_id')->
                        where('promotes.ac_id',$cur_acyear->ac_id);
        if($course!=''){
            $query->where('promotes.course_id',$course);
        }
        if($semester!=''){
            $query->where('promotes.semester_id',$semester);
        }
        if(!empty($request->input('search.value'))){
            $this->search = $request->input('search.value');
            $query->where(function($qry) {
                return $qry->where('adno','like','%'.$this->search.'%')->
                orWhere('name','like','%'.$this->search.'%')->
                orWhere('course_name','like','%'.$this->search.'%')->
                orWhere('semester_name','like','%'.$this->search.'%');
            });
        }
        $total_filtered = $query->count();
        $students  =   $query->select('students.*','courses.course_name','semesters.semester_name')->
                    offset($start)->limit($limit)->groupBy('promotes.student_id')->orderBy($order,$dir)->get();
                 
        $data = array();
        if($students){
            foreach($students as $student){
                $row['adno'] = $student->adno;
                $row['name'] = $student->name;
                $row['course'] = $student->course_name;
                $row['semester'] = $student->semester_name;
                
                $fees = Fee::studentFees($student->student_id,$cdate);
                $famount=0;
                foreach($fees as $ac_id => $sem_details){
                    foreach($sem_details as $sem_id => $term_details){
                    foreach($term_details['fee_structure'] as $term_id => $fee_details){
                        foreach($fee_details['fee_type'] as $ftype_id => $ftype_details){
                            $famount += $ftype_details['due_amount']+$ftype_details['fine_amount'];
                           
                        }
                    }
                }
                }
                $row['due'] = number_format($famount,2);
                $row['duedate']=date('d-m-Y',strtotime($ftype_details['end_date']));
                $html="";
                if(MenuAccess::have_access(39) == true){
                    $html .= '<a href="'.route('student.due',encrypt($student->student_id)).'" title="View" target="blank_"><i class="icon-sm icon-magnifier-add text-primary"></i></a>';
                }
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
    }
    
    public function due_total_ajax(Request $request){
        
        $course = $request->get('course');
        $semester = $request->get('semester');
        $cdate = $request->get('cdate');
        
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        
        $query  =   Student::join('promotes','students.student_id','=','promotes.student_id')->
                        join('courses','promotes.course_id','=','courses.course_id')->
                        join('semesters','promotes.semester_id','=','semesters.semester_id')->
                        where('promotes.ac_id',$cur_acyear->ac_id);
        if($course!=''){
            $query->where('promotes.course_id',$course);
        }
        if($semester!=''){
            $query->where('promotes.semester_id',$semester);
        }
        $total=0;
        $students  =   $query->get();
        if($students){
            foreach($students as $student){
                $fees = Fee::studentFees($student->student_id,$cdate);
                $famount=0;
                foreach($fees as $ac_id => $sem_details){
                    foreach($sem_details as $sem_id => $term_details){
                    foreach($term_details['fee_structure'] as $term_id => $fee_details){
                        foreach($fee_details['fee_type'] as $ftype_id => $ftype_details){
                            $famount += $ftype_details['due_amount']+$ftype_details['fine_amount'];
                             
                            
                             
                           
                        }
                    }
                }
                }
               $total +=$famount;
            }
        }
        echo number_format($total,2);
    }
    
    public function student_due($sid){
        $cdate = date('Y-m-d');
        $student = Student::find(decrypt($sid));
        $fees = Fee::studentFees(decrypt($sid),$cdate);
        
        return view('fees.student_due',compact('student','fees'));
    }
    
    public function otherfee()
    {
        $acyears = Acyear::all();
        $rec_no = Otherfee::max('receipt_no') + 1;
        $paymentmode = Paymentmode::all();
        $feetype = FeeType::all();
        
        return view('fees.otherfee',compact('acyears','rec_no','paymentmode','feetype'));
    }
    
    public function otherfee_save(Request $request)
    {
        $validated = $request->validate([
            'acyear' => 'required|exists:acyears,ac_id',
            'date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'receipt_number' => 'required|numeric',
            'student_name' => 'required|string',
            'address' => 'nullable|string',
            'payment_mode' => 'required|exists:paymentmodes,mode_id',
            'types' => 'required|array',
            'amounts' => 'required|array',
            'remarks' => 'nullable|string',
        ]);
            
        $feetypeData = Feetype::pluck('ledger_id', 'type_id')->toArray();
            
        DB::beginTransaction();
            
        try {
            $rec_no = Otherfee::max('receipt_no') + 1;
            $fgroup_id = Otherfee::withTrashed()->max('group_id') + 1 ?? 1;
            $data = [
                'group_id' => $fgroup_id,
                'receipt_no' => $rec_no,
                'ac_id' => $validated['acyear'],
                'student_name' => $validated['student_name'],
                'address' => $validated['address'],
                'paid_date' => $validated['date'],
                'mode_id' => $validated['payment_mode'],
                'remarks' => $validated['remarks'],
                'account_id' => $this->accountId,
                'created_by' => auth()->user()->user_id,
            ];
                
            $i = 0; $vouchers = [];
            foreach ($validated['types'] as $key => $typeid) {
                $data['type_id'] = $typeid;
                $data['fee_amount'] = $validated['amounts'][$key];
                
                if($data['fee_amount'] != '' && $data['fee_amount'] > 0){
                    // Create the fee record in the database
                    Otherfee::create($data);
                    
                    $ledger_id = $feetypeData[$typeid] ?? null;
                    if($ledger_id){
                        if (isset($vouchers[$ledger_id])) {
                            $vouchers[$ledger_id] += $data['fee_amount'];
                        }else{
                            $vouchers[$ledger_id] = $data['fee_amount'];
                        }
                    }
                    print_r($data);
                    $i++;
                }
            }
            
            if($i>0){
                if(!empty($vouchers)){
                    $group_id = Voucher::withTrashed()->max('group_id') + 1 ?? 1;
                    $vno = Voucher::where('vtype_id', 2)
                          ->where('account_id', $this->accountId)
                          ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                          ->max('vno') + 1 ?? 1;
                        
                    $vdata = [
                        'group_id' => $group_id,
                        'vtype_id' => 2,
                        'vno' => $vno,
                        'vdate' => $data['paid_date'],
                        'dr_ledger' => $data['mode_id'],
                        'table_name' => 'otherfees',
                        'table_group_id' => $fgroup_id,
                        'account_id' => $this->accountId,
                        'created_by' => auth()->user()->user_id
                    ];
                    
                    $k=1;
                    foreach($vouchers as $key => $value){
                        $vdata['vorder'] = $k;
                        $vdata['cr_ledger'] = $key;
                        $vdata['vamount'] = $value;
                        
                        $vsave = Voucher::create($vdata);
                        if($vsave){
                            $k++;
                        }
                    }
                }
            }
            
            DB::commit();
            
            echo "<script>window.open('".route('otherfee.print', encrypt($fgroup_id))."');</script>";
            return redirect()->route('otherfee')->with('success', 'Fee Paid Successfully!');
        } catch (\Exception $e) {
            DB::rollBack();
                
            \Log::error('Error saving payment: ' . $e->getMessage());
                
            return redirect()->route('otherfee')->with('error', 'An error occurred while processing the payment.');
        }
    }
    
    public function otherfee_report()
    {
        $otherfees=StudentOtherFees::all();
     $acyears=Acyear::all();
       return view('fees.otherreport',compact('otherfees','acyears'));
      
  }
  
  public function otherfee_ajax(Request $request)
  {
      $validated=$request->validate([
          'fdate'=>'required',
          'tdate'=>'required',
          ]);
          
          $fdate=$request->fdate;
          $tdate=$request->tdate;
          $otherfees=StudentOtherFees::whereBetween('date',[$fdate,$tdate])->get();
         
          return view('fees.otherreport',compact('otherfees','fdate','tdate'));
  }
    
    
   public function otherreport_cancel($othrfee_id)
   {
       
       
       
       
       try {
        $otherFee = StudentOtherFees::findOrFail($othrfee_id);
        $otherFee->status = 1;
        $otherFee->save();

        return true;
    } catch (\Exception $e) {
        return response()->json(['success' => false, 'error' => $e->getMessage()]);
    }
        // $otherfee = StudentOtherFees::where('othrfee_id',$feeid)->first();
         $otherfee = StudentOtherFees::find($request->other_fee_id);
       
   

       
    } 
     
   
 public function otherfeeprint($id)
 {
      $ins_det = instituteDetails();
     $otherfeee=StudentOtherFees::where('othrfee_id',$id)->first();
     $feetypamount=FeetypeAmount::where('othrfee_id',$id)->get();
     return view('fees.otherfeeprint',compact('otherfeee','ins_det','feetypamount'));
 }
 
  public function otherfeepdf($id)
  {
       $ins_det = instituteDetails();
        $feetypamount=FeetypeAmount::where('othrfee_id',$id)->get();
        $other = StudentOtherFees::findOrFail($id);
        $data =[
            'othrfee_id'=>$other,
            
            ];
            
              $pdf = PDF::loadView('fees.otherfeepdf',compact('other','ins_det','feetypamount'));
               $pdf->setPaper('A4', 'portrait');
               return $pdf->download('otherfeepdf.pdf');
  }
    
    
    
    public function student_fee_structure()
    {
        $acyears=Acyear::all();
        $courses=Course::all();
        $categories = FeeCategory::all();
        $terms = FeeTerm::all();
        return view('fees.student_fee_structure',compact('acyears','courses','categories','terms'));
    }
    public function student_fee_structure_get(Request $request)
    {
        $acyear=Acyear::find($request->aid);
        $crs=Course::find($request->cid);
        $sem=Semester::find($request->sid);
        $student=Student::find($request->stuid);
        $cat_det = FeeCategory::find($request->ctid);
       
        $term_det = FeeTerm::find($request->tid);
         
        $student=Student::join('promotes','promotes.student_id','=','promotes.student_id')
        ->join('semesters','semesters.semester_id','=','promotes.semester_id')
        ->where('promotes.course_id',$request->cid)
        ->where('promotes.semester_id',$request->sid)
        ->where('promotes.ac_id',$request->aid)
        ->where('students.student_id',$request->stuid)
        ->select('students.*','semesters.semester_name')
        ->first();
        
         $feetypes = FeeType::all();
        
        $stutypes=StudentFeeStuType::join('student_fee_structure','student_feestructure_types.stu_structure_id','=','student_fee_structure.stu_structure_id')
        ->where(array('student_fee_structure.ac_id'=>$request->aid,'student_fee_structure.course_id'=>$request->cid,'student_fee_structure.semester_id'=>$request->sid,'student_fee_structure.category_id'=>$request->ctid,'student_fee_structure.term_id'=>$request->tid,'student_fee_structure.student_id'=>$request->stuid))
        ->select('student_feestructure_types.*')
        ->latest()
        ->get();
       
        if ($stutypes->isEmpty()) {
        $stypes = FeeStructureType::join('fee_structures', 'fee_structure_types.structure_id', '=', 'fee_structures.structure_id')
            ->where([
                'fee_structures.ac_id' => $request->aid,
                'fee_structures.course_id' => $request->cid,
                'fee_structures.semester_id' => $request->sid,
                'fee_structures.category_id' => $request->ctid,
                'fee_structures.term_id' => $request->tid
            ])
            ->select('fee_structure_types.*')
            ->latest()
            ->get();
    } else {
        $stypes = collect(); 
    }
        
       

        


        
       
                        return view('fees.student_fee_structure_get',compact('cat_det','term_det','student','feetypes','stypes','stutypes','sem','crs','acyear'));
                        
    }
    
    
    
    
    public function student_fee_structure_save(Request $request)
    {
      $validated = $request->validate([
            'ac_id' => 'required',
            'course_id' => 'required',
            'semester_id' => 'required',
            'category_id' => 'required',
            'term_id' => 'required',
        ]);
        
      $type = $request->type;
        $amount = $request->amount;
        $start = $request->start;
        $end = $request->end;
        $in_count=0;
         $studentfeestructure = StudentFeeStru::firstOrCreate([
            'ac_id' => $request->ac_id,
            'course_id' => $request->course_id,
            'semester_id' => $request->semester_id,
            'category_id' => $request->category_id,
            'term_id' => $request->term_id,
            'student_id' => $request->student_id,
        ]);
         if($studentfeestructure){
            foreach($type as $key=>$value){
                if($amount[$key] > 0 && $start[$key]!='0000-00-00' && $start[$key]!='1970-01-01' && $end[$key]!='0000-00-00' && $end[$key]!='1970-01-01'){
                    $fstype = StudentFeeStuType::updateOrCreate([
                        'stu_structure_id' => $studentfeestructure->stu_structure_id,
                        'type_id' => $value],[
                        'amount' => $amount[$key],
                        'start_date' => $start[$key],
                        'end_date' => $end[$key]
                    ]);
                    
                    if($fstype){ $in_count++; }
                }
            }
        }
    if($in_count > 0){
            $msg_array = array('message'=>'Fee Structure Saved Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Fee Structure Could not be Saved !','alert-type'=>'error');
        }
        
        return redirect()->route('student.fee.structure')->with($msg_array);
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
}
