<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use App\Models\User;
use App\Models\Acyear;
use App\Models\Course;
use App\Models\Semester;
use App\Models\Student;
use App\Models\Hostel;
use App\Models\HostelMonth;
use App\Models\HostelFee;
use App\Models\HostelFacility;
use App\Models\HostelPayment;
use App\Models\Paymentmode;
use App\Models\MenuAccess;

use Illuminate\Support\Facades\DB;

class Hostels extends Controller
{
    public function payment(){
        $students = Student::all();
        $paymodes = Paymentmode::where('status',1)->get();
        return view('hostel.payment',compact('students','paymodes'));
    }
    
    public function get_payment(Request $request){
        
        $validated = $request->validate([
                'student_id' => 'required',
                'paid_date' => 'required|date',
                'mode_id' => 'required',
            ]);
        $fees = array();
        $student_id = $request->student_id;
        $paid_date = $request->paid_date;
        $mode_id = $request->mode_id;
        $students = Student::all();
        $paymodes = Paymentmode::where('status',1)->get();
        $student_det = Student::find($student_id);
        if($paid_date <= date('Y-m-d')){
            $fees = HostelPayment::hostelFees($student_id);
        }else{
            $msg_array = array('message'=>'Invalid Date !','alert-type'=>'error');
        }
         
        return view('hostel.get_payment',compact('students','student_id','paid_date','paymodes','mode_id','fees','student_det'));
    }
    
    public function save_payment(Request $request){
        
        $validated = $request->validate([
                'posted' => 'present|array'
            ]);
        $data=array();
        $rec_no = HostelPayment::max('receipt_no');
        $rec_no += 1;
        $data['receipt_no'] = $rec_no;
        $data['student_id'] = decrypt($request->sid);
        $data['paid_date'] = $request->pdate;
        $data['mode_id'] = $request->mode;
        $data['created_by'] = auth()->user()->user_id;
        $i=0;   
        foreach($request->posted as $ids){
            $fee_det = explode('-',$ids);
            $data['month'] = $fee_det[0];
            $data['year'] = $fee_det[1];
            $data['ptype'] = $fee_det[2];
            $data['semester_id'] = $request->input("sem".$ids);
            $data['amount'] = $request->input("fee_amount".$ids);
            $disc=0;
            if($request->input("disc_amount".$ids)!=''){ $disc = $request->input("disc_amount".$ids); }
            $data['discount'] = $disc;
            $data['remarks'] = $request->input("remarks".$ids);
            
            $fees = HostelPayment::create($data);
            if($fees){ $i++; }
        }
        
        if($i > 0){
            echo "<script>window.open('".route('hostelpayment.print',encrypt($rec_no))."');</script>";
            $msg_array = array('message'=>'Hostel Fee Paid Successfully !','alert-type'=>'success');
        }else{
            $msg_array = array('message'=>'Hostel Fee Could not be Paid !','alert-type'=>'error');
        }
        
        return redirect()->route('hostelpayment')->with($msg_array);
    }
    
    public function report(){
        
        $paymodes = Paymentmode::where('status',1)->get();
        $staffs = User::all();
        $courses = Course::all();
        
        return view('hostel.report',compact('courses','paymodes','staffs'));
    }
    
    public function report_ajax(Request $request){
        
        $col_order = ['receipt_no','paid_date','adno','name','course_name','semester_name',null,'uname',null];
        $total_data = HostelPayment::whereNull('deleted_at')->groupBy('receipt_no')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        
        $course = $request->get('course');
        $semester = $request->get('semester');
        $fdate = $request->get('fdate');
        $tdate = $request->get('tdate');
        $mode = $request->get('mode');
        $ftype = $request->get('ftype');
        $staff = $request->get('staff');
        
        $query = HostelPayment::join('students','hostel_payments.student_id','=','students.student_id')->
                    join('courses','students.course_id','=','courses.course_id')->
                    join('semesters','hostel_payments.semester_id','=','semesters.semester_id')->
                    join('users','hostel_payments.created_by','=','users.user_id')->
                    where('hostel_payments.status',1)->
                    whereNull('hostel_payments.deleted_at');
        if($fdate!=''){ $query->where('hostel_payments.paid_date','>=',$fdate); }
        if($tdate!=''){ $query->where('hostel_payments.paid_date','<=',$tdate); }
        if($course!=''){ $query->where('students.course_id',$course); }
        if($semester!=''){ $query->where('hostel_payments.semester_id',$semester); }
        if($mode!=''){ $query->where('hostel_payments.mode_id',$mode); }
        if($ftype!=''){ $query->where('hostel_payments.ptype',$ftype); }
        if($staff!=''){ $query->where('hostel_payments.created_by',$staff); }
        
        if(!empty($request->input('search.value'))){
            $this->search = $request->input('search.value');
            $query->where(function($qry) {
                return $qry->orWhere('hostel_payments.receipt_no','like','%'.$this->search.'%')->
                orWhere('students.adno','like','%'.$this->search.'%')->
                orWhere('students.name','like','%'.$this->search.'%')->
                orWhere('courses.course_name','like','%'.$this->search.'%')->
                orWhere('semesters.semester_name','like','%'.$this->search.'%')->
                orWhere('users.name','like','%'.$this->search.'%');
            });
        };
        $total_filtered = $query->groupBy('hostel_payments.receipt_no')->count();
        $fees  =   $query->select('hostel_payments.receipt_no','hostel_payments.paid_date','students.adno','students.name',DB::raw('sum(hostel_payments.amount) as amt'),DB::raw('sum(hostel_payments.discount) as disc'),'courses.course_name','semesters.semester_name','users.name as uname')->
                    offset($start)->limit($limit)->groupBy('hostel_payments.receipt_no')->orderBy($order,$dir)->get();
                 
        $data = array();
        if($fees){
            foreach($fees as $fee){
                $row['recno'] = $fee->receipt_no;
                $row['date'] = date('d/m/Y',strtotime($fee->paid_date));
                $row['adno'] = $fee->adno;
                $row['name'] = $fee->name;
                $row['course'] = $fee->course_name;
                $row['semester'] = $fee->semester_name;
                $row['amount'] = number_format(($fee->amt-$fee->disc),2);
                $row['collected'] = $fee->uname;
                $html="";
                if(MenuAccess::have_access(47) == true){
                    $html .= '<a href="'.route('hostelpayment.print',encrypt($fee->receipt_no)).'" title="Print" target="blank_"><i class="icon-sm icon-printer text-dark"></i></a>&nbsp;&nbsp;';
                }
                if(MenuAccess::have_access(48) == true){
                    $html .= '<a href="'.route('hostelpayment.delete',encrypt($fee->receipt_no)).'" title="Delete" onClick="return confirm(\'Are you sure you want to delete?\');" ><i class="icon-sm icon-trash text-danger"></i></a>';
                }
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
    }
    
    public function total_ajax(Request $request){
        
        $course = $request->get('course');
        $semester = $request->get('semester');
        $fdate = $request->get('fdate');
        $tdate = $request->get('tdate');
        $mode = $request->get('mode');
        $ftype = $request->get('ftype');
        $staff = $request->get('staff');
        
        $amount=0;
        
        $query = HostelPayment::join('students','hostel_payments.student_id','=','students.student_id')->
                    join('courses','students.course_id','=','courses.course_id')->
                    join('semesters','hostel_payments.semester_id','=','semesters.semester_id')->
                    join('users','hostel_payments.created_by','=','users.user_id')->
                    where('hostel_payments.status',1)->
                    whereNull('hostel_payments.deleted_at');
                    if($fdate!=''){ $query->where('hostel_payments.paid_date','>=',$fdate); }
                    if($tdate!=''){ $query->where('hostel_payments.paid_date','<=',$tdate); }
                    if($course!=''){ $query->where('students.course_id',$course); }
                    if($semester!=''){ $query->where('hostel_payments.semester_id',$semester); }
                    if($mode!=''){ $query->where('hostel_payments.mode_id',$mode); }
                    if($ftype!=''){ $query->where('hostel_payments.ptype',$ftype); }
                    if($staff!=''){ $query->where('hostel_payments.created_by',$staff); }
                    
        $data = $query->select(DB::raw("sum(hostel_payments.amount-hostel_payments.discount) as fees"))->get()->first();
        if(!empty($data)){
            $amount = $data->fees;
        }
        
        echo number_format($amount,2);
    }
    
    public function delete($recno){
        
        $fee = HostelPayment::where('receipt_no',decrypt($recno));
        if($fee){
            if($fee->delete()){
                $msg_array = array('message'=>'Receipt Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Receipt Could not be Deleted !','alert-type'=>'error');
            }
        }
        
        return redirect()->route('hostelpayment.report')->with($msg_array);
    }
    
    public function print_bill($recno){
        
        $ins_det = instituteDetails();
        $fee_det = HostelPayment::where('receipt_no',decrypt($recno))->whereNull('deleted_at')->first();
        $fee_item_det = HostelPayment::where('receipt_no',decrypt($recno))->whereNull('deleted_at')->get();
         
        return view('hostel.print_bill',compact('fee_det','fee_item_det','ins_det'));
    }
    
    public function due_report(){
        
        $courses = Course::all();
        
        return view('hostel.due_report',compact('courses'));
    }
    
    public function due_ajax(Request $request){
        
        $col_order = ['adno','name','course_name','semester_name','',''];
        $total_data = Student::whereNull('deleted_at')->count();
        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $col_order[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        
        $course = $request->get('course');
        $semester = $request->get('semester');
        $ftype = $request->get('ftype');
        $cdate = $request->get('cdate');
        
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        
        $query  =   Student::join('promotes','students.student_id','=','promotes.student_id')->
                        join('courses','promotes.course_id','=','courses.course_id')->
                        join('semesters','promotes.semester_id','=','semesters.semester_id')->
                        where('promotes.ac_id',$cur_acyear->ac_id);
        if($course!=''){
            $query->where('promotes.course_id',$course);
        }
        if($semester!=''){
            $query->where('promotes.semester_id',$semester);
        }
        if(!empty($request->input('search.value'))){
            $this->search = $request->input('search.value');
            $query->where(function($qry) {
                return $qry->where('adno','like','%'.$this->search.'%')->
                orWhere('name','like','%'.$this->search.'%')->
                orWhere('course_name','like','%'.$this->search.'%')->
                orWhere('semester_name','like','%'.$this->search.'%');
            });
        };
        $total_filtered = $query->count();
        $students  =   $query->select('students.*','courses.course_name','semesters.semester_name')->
                    offset($start)->limit($limit)->groupBy('promotes.student_id')->orderBy($order,$dir)->get();
                 
        $data = array();
        if($students){
            foreach($students as $student){
                $row['adno'] = $student->adno;
                $row['name'] = $student->name;
                $row['course'] = $student->course_name;
                $row['semester'] = $student->semester_name;
                
                $fees = HostelPayment::hostelFees($student->student_id);
                $famount=0;
                foreach($fees as $ac_id => $ac_details){
                    foreach($ac_details as $month_id => $month_details){
                        foreach($month_details['fee'] as $ftype_id => $fee_details){
                            if($ftype!=''){
                                if($ftype==$ftype_id){
                                    $famount += $fee_details['due'];
                                }
                            }else{
                                $famount += $fee_details['due'];
                            }
                        }
                    }
                }
                
                $row['due'] = number_format($famount,2);
                $html="";
                if(MenuAccess::have_access(49) == true){
                    $html .= '<a href="'.route('hostel.student.due',encrypt($student->student_id)).'" title="View" target="blank_"><i class="icon-sm icon-magnifier-add text-primary"></i></a>';
                }
                $row['action'] = $html;
                
                $data[] = $row;
            }
        }
        
        $json = array(
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data
        );
            
        echo json_encode($json);
    }
    
    public function due_total_ajax(Request $request){
        
        $course = $request->get('course');
        $semester = $request->get('semester');
        $ftype = $request->get('ftype');
        $cdate = $request->get('cdate');
        
        $cur_acyear = Acyear::where('default1',1)->whereNull('deleted_at')->first();
        
        $query  =   Student::join('promotes','students.student_id','=','promotes.student_id')->
                        join('courses','promotes.course_id','=','courses.course_id')->
                        join('semesters','promotes.semester_id','=','semesters.semester_id')->
                        where('promotes.ac_id',$cur_acyear->ac_id);
        if($course!=''){
            $query->where('promotes.course_id',$course);
        }
        if($semester!=''){
            $query->where('promotes.semester_id',$semester);
        }
        $total=0;
        $students  =   $query->get();
        if($students){
            foreach($students as $student){
                $fees = HostelPayment::hostelFees($student->student_id);
                $famount=0;
                foreach($fees as $ac_id => $ac_details){
                    foreach($ac_details as $month_id => $month_details){
                        foreach($month_details['fee'] as $ftype_id => $fee_details){
                            if($ftype!=''){
                                if($ftype==$ftype_id){
                                    $famount += $fee_details['due'];
                                }
                            }else{
                                $famount += $fee_details['due'];
                            }
                        }
                    }
                }
                $total+=$famount;
            }
        }
        echo number_format($total,2);
    }
    
    public function student_due($sid){
        $cdate = date('Y-m-d');
        $student = Student::find(decrypt($sid));
        $fees = HostelPayment::hostelFees(decrypt($sid));
        
        return view('hostel.student_due',compact('student','fees'));
    }
}
