<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\DB;

class HostelPayment extends Model
{
    use HasFactory, SoftDeletes;
    
    protected $table = 'hostel_payments';
    
    protected $primaryKey = 'fee_id'; 
    
    protected $guarded = [];
    
    protected $hidden = ['fee_id'];
    
    public function student(){
        return $this->hasOne(Student::class,'student_id','student_id');
    }
    
    public function mode(){
        return $this->hasOne(Paymentmode::class,'mode_id','mode_id');
    }
    
    public function getPtypeTextAttribute(){
        if($this->ptype=='1') return "Rent";
        else if($this->ptype=='2') return "Mess Fee";
    }
    
    public function getPaidDateFormattedAttribute(){
        return date('d/m/Y',strtotime($this->paid_date));
    }
    
    public function getMonthFormattedAttribute(){
        return date('F Y',strtotime($this->year.'-'.$this->month.'-01'));
    }
    
    protected $appends = ['ptype_text','paid_date_formatted','month_formatted'];
    
    public static function studentAcyear($sid){
        $acyears=array();
        $rows = Promote::where('student_id',$sid)->whereNull('deleted_at')->get();
        if(!empty($rows)){
            foreach($rows as $row){
                $acyears[$row->ac_id] = array('course'=>$row->course_id,'semester'=>$row->semester_id);
            }
        }
        return $acyears;
    }
    
    public static function hostelFees($sid){
        $fs_array = array();
		$acyear = self::studentAcyear($sid);
		$facility = HostelFacility::where('student_id',$sid)->where('status',1)->whereNull('deleted_at')->first();
		if($facility){
    		foreach($acyear as $ac_id=>$ac_det){
    		    $month_array = HostelMonth::where('ac_id',$ac_id)->whereNull('deleted_at')->get();
        		foreach($month_array as $month){
        		    $edate = date('Y-m-t',strtotime($month->year.'-'.$month->month.'-01'));
        		    if($facility->from<=$edate){
            			$hostel_fee_structure = self::hostelFeeStructure($sid,$facility->hostel_id,$month->month_id,$month->month,$month->year);
            			if(count($hostel_fee_structure) > 0){
            				$fs_array[$ac_id][$month->month_id] = array('semester'=>$ac_det['semester'],'month'=>$month->month,'year'=>$month->year,'myear'=>date('F Y',strtotime($edate)),'fee'=>$hostel_fee_structure);
            			}
        		    }
        		}
    		}
		}
		return $fs_array;
    }
    
    public static function hostelFeeStructure($sid,$hostel,$mid,$month,$year){
        $pdate = date('Y-m-d',strtotime($year.'-'.$month.'-01'));
		$fee_array = array();
		$rent = HostelFee::where('hostel_id',$hostel)->where('from','<=',$pdate)->whereNull('deleted_at')->orderBy('from','DESC')->first();
		if(!empty($rent)){
		    if($rent->amount!=NULL && $rent->amount>0){
    		    $rent_due = self::hostelDue($sid,$month,$year,$rent->amount);
    		    if($rent_due>0){
    		        $fee_array[1] = array('ptype'=>1,'fee_type'=>'Rent','fee'=>$rent->amount,'due'=>$rent_due);
    		    }
		    }
		}

		$mess = MessFee::where('hostel_id',$hostel)->where('month_id',$mid)->whereNull('deleted_at')->orderBy('month_id','DESC')->first();
		if($mess){
		    if($mess->amount!=NULL && $mess->amount>0){
    		    $mess_due = self::messDue($sid,$month,$year,$mess->amount);
    		    if($mess_due>0){
    		        $fee_array[2] = array('ptype'=>2,'fee_type'=>'Mess Fee','fee'=>$mess->amount,'due'=>$mess_due);
    		    }
		    }
		}
		
		return $fee_array;
    }
    
    public static function hostelDue($sid,$month,$year,$amount){
	    $due_amount=0;
	    $paid = self::where(array('student_id'=>$sid,'ptype'=>1,'month'=>$month,'year'=>$year,'status'=>1))->whereNull('deleted_at')->sum('amount');
		if($paid){
		    $due_amount = abs($amount-$paid);
		}else{
		    $due_amount=$amount;
		}
		return $due_amount;
	}
	
	public static function messDue($sid,$month,$year,$amount){
		$due_amount=0;
		$paid = self::where(array('student_id'=>$sid,'ptype'=>2,'month'=>$month,'year'=>$year,'status'=>1))->whereNull('deleted_at')->sum('amount');
		if($paid){
		    $due_amount = abs($amount-$paid);
		}else{
		    $due_amount=$amount;
		}
		return $due_amount;
	}
}
