<?php

namespace App\Models;
use Illuminate\Support\Facades\DB;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $table = 'users';
    protected $primaryKey = 'user_id'; 
    
    protected $guarded = [];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'user_id',
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];
    
    public function utype(){
        return $this->hasOne(UserType::class,'type_id','type_id');
    }
    
    public function getStatusTextAttribute(){
        if($this->status==1){ return "Active"; }
        else if($this->status==1){ return "Inactive"; }
    }
    
    protected $appends = ['status_text'];
    
    public function desig()
    {
        return $this->hasOne(UserType::class,'type_id','type_id');
    }
    
  public function slry()  
    {
        return $this->hasmany(BasicSalary::class,'user_id','user_id');
    }
   public function leavset()
   {
       return $this->hasOne(Staffleaveset::class,'leav_id','leav_id');
   }
   
     public function salaryprocess(){
        return $this->hasmany(SalaryProcessing::class,'user_id','user_id');
    }
    
    public function advancesalprocess(){
        return $this->hasmany(AdvanceSalaryProcess::class,'user_id','user_id');
    }
    
    public function salaryad(){
        return $this->hasmany(StaffAllowDedu::class,'user_id','user_id');
    }
    
    
    public function totalWorkingDays($user_id,$monthYear)
{
    $workingDays = 0;
    [$year, $month] = explode('-', $monthYear);

   
    $staff = User::select('type_id', 'dateof_join')->where('user_id', $user_id)->first();
    if (!$staff) {
        return 0; 
    }
    
    $staffType = $staff->type_id;
    $joiningDate = $staff->dateof_join;

  
    $paymentCriteria = PaymentCriteria::first()->payment_criteria;

    if ($paymentCriteria == 'Pay per Month') {
        // Payment criteria = 0
        $daysInMonth = cal_days_in_month(CAL_GREGORIAN, $month, $year);
        for ($i = 1; $i <= $daysInMonth; $i++) {
            $date = "$year-$month-$i";
            $dayOfWeek = date('D', strtotime($date));

            /*$isHoliday = MasterHoliday::where('staff_holiday', '1')
                ->where('designation', 'like', "%@$staffType@%")
                ->where('leave_date', $date)
                ->exists();

            if ($dayOfWeek != 'Sun' && !$isHoliday) {
                $workingDays++;
            }*/
            
             if ($dayOfWeek != 'Sun'){ $workingDays++; }
        }
    } else {
        // Payment criteria = 1
        $joinDay = (int)date('d', strtotime($joiningDate));
        $preDate = "$year-$month-$joinDay";
        $nextDate = date("Y-m-d", strtotime("$preDate +1 month"));
        
        $preDays = cal_days_in_month(CAL_GREGORIAN, $month, $year);

      
        for ($i = $joinDay; $i <= $preDays; $i++) {
            $date = "$year-$month-$i";
            $dayOfWeek = date('D', strtotime($date));

           /* $isHoliday = MasterHoliday::where('staff_holiday', '1')
                ->where('designation', 'like', "%@$staffType@%")
                ->where('leave_date', $date)
                ->exists();

            if ($dayOfWeek != 'Sun' && !$isHoliday) {
                $workingDays++;
            }*/
            if ($dayOfWeek != 'Sun')
            {
                $workingDays++;
            }
        }

       
        $nextMonth = date("m", strtotime($nextDate));
        $nextYear = date("Y", strtotime($nextDate));
        for ($j = 1; $j < $joinDay; $j++) {
            $date = "$nextYear-$nextMonth-$j";
            $dayOfWeek = date('D', strtotime($date));

            /*$isHoliday = MasterHoliday::where('staff_holiday', '1')
                ->where('designation', 'like', "%@$staffType@%")
                ->where('leave_date', $date)
                ->exists();

            if ($dayOfWeek != 'Sun' && !$isHoliday) {
                $workingDays++;
            }*/
             if ($dayOfWeek != 'Sun') {
                 $workingDays++;
            }
        }
    }

    return $workingDays; 
}

 public function totalPresentDays($staffId, $monthYear)
    {
        $presentDays = 0;
        [$year, $month] = explode("-", $monthYear);

       
        $paymentCriteria = DB::table('set_payment_criteria')->first()->payment_criteria;

        if ($paymentCriteria == 'Pay per Month') {
          
            $daysInMonth = cal_days_in_month(CAL_GREGORIAN, $month, $year);
            for ($i = 1; $i <= $daysInMonth; $i++) {
                $nowDate = "$year-$month-$i";    
                $attendance = DB::table('staff_attendance')
                    ->select('am', 'pm', 'leav_id')
                    ->where('date', $nowDate)
                    ->where('user_id', $staffId)
                    ->first();

                if ($attendance) {
                    $am = $attendance->am;
                    $pm = $attendance->pm;
                    $leaveType = $attendance->leav_id;

                    if ($leaveType == 1) {
                        if (($am == 0 && $pm == 1) || ($am == 1 && $pm == 0)) {
                            $presentDays += 0.5;
                        }
                    } else {
                        if ($am != 1 && $pm != 1) {
                            $presentDays++;
                        }
                    }                    
                }
            }
        } else {
           
            $joiningDate = DB::table('users')->where('user_id', $staffId)->value('dateof_join');
            $joinDay = (int) date('d', strtotime($joiningDate));
            $preDate = "$year-$month-$joinDay";
            $nextDate = date("Y-m-d", strtotime("$preDate +1 month"));
            $nextMonth = date("m", strtotime($nextDate));
            $nextYear = date("Y", strtotime($nextDate));
            $daysInMonth = cal_days_in_month(CAL_GREGORIAN, $month, $year);
            
           
            for ($i = $joinDay; $i <= $daysInMonth; $i++) {
                $nowDate = "$year-$month-$i";    
                $attendance = DB::table('staff_attendance')
                    ->select('am', 'pm', 'leav_id')
                    ->where('date', $nowDate)
                    ->where('user_id', $staffId)
                    ->first();

                if ($attendance) {
                    $am = $attendance->am;
                    $pm = $attendance->pm;
                    $leaveType = $attendance->leav_id;

                    if ($leaveType == 1) {
                        if (($am == 0 && $pm == 1) || ($am == 1 && $pm == 0)) {
                            $presentDays += 0.5;
                        }
                    } else {
                        if ($am != 1 && $pm != 1) {
                            $presentDays++;
                        }
                    }                    
                }
            }

            
            for ($j = 1; $j < $joinDay; $j++) {
                $nowDate = "$nextYear-$nextMonth-$j";    
                $attendance = DB::table('staff_attendance')
                    ->select('am', 'pm', 'leav_id')
                    ->where('date', $nowDate)
                    ->where('user_id', $staffId)
                    ->first();

                if ($attendance) {
                    $am = $attendance->am;
                    $pm = $attendance->pm;
                    $leaveType = $attendance->leav_id;

                    if ($leaveType == 1) {
                        if (($am == 0 && $pm == 1) || ($am == 1 && $pm == 0)) {
                            $presentDays += 0.5;
                        }
                    } else {
                        if ($am != 1 && $pm != 1) {
                            $presentDays++;
                        }
                    }                    
                }
            }
        }

        return $presentDays; 
    }
    
    
   public function netSalary($basicPay,$advanceRecovery, $allAmount, $dedAmount,$leaveStatus,$workingDays = null,$presentDays = null,$monthYear = null) 
   {
        $netSalary = 0;

        if ($leaveStatus == 1) {
           
            [$salaryYear, $salaryMonth] = explode("-", $monthYear);
            $daysInMonth = cal_days_in_month(CAL_GREGORIAN, $salaryMonth, $salaryYear);
            $salaryPerDay = $basicPay / $daysInMonth;

            $leaveDays = abs($workingDays - $presentDays);
            $salaryLoss = $leaveDays * $salaryPerDay;
            $currentBasicPay = max(0, $basicPay - $salaryLoss); 

           
            $advanceRecovery = min($advanceRecovery, $currentBasicPay);
            
            
            $netSalary = round(($currentBasicPay + $allAmount) - ($dedAmount + $advanceRecovery));
        } else { 
            
            $advanceRecovery = min($advanceRecovery, $basicPay);
            
            
            $netSalary = round(($basicPay + $allAmount) - ($dedAmount + $advanceRecovery));
        }

        $advanceRecovery = round($advanceRecovery);
        return [$netSalary, $advanceRecovery];
    } 
    
}
