@extends('layouts.template')
@section('title','Ledger')
@section('heading','Ledgers')
@section('breadcrumb','Home / Ledgers')
@section('content')
    @if($add==1)
        <div class="card card-primary">
            <div class="card-header"><b>Add Ledger</b></div>
            <div class="card-body">
                <form action="{{ route('accounts.ledger.save') }}" method="post">
                    @csrf
                    <div class="row">
                        <div class="col-12 col-sm-4">
                            <div class="form-group">
                                <label for="ledger_name" class="required">Ledger Name</label>
                                <input type="text" class="form-control @error('ledger_name') border-danger @enderror" id="ledger_name" name="ledger_name" value="{{ old('ledger_name') }}">
                                @error('ledger_name')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                            </div>
                        </div>
                        <div class="col-12 col-sm-4">
                            <div class="form-group">
                                <label for="acsubgrp_id" class="required">Group of</label>
                                <select class="form-control" id="acsubgrp_id" name="acsubgrp_id" required>
                                    @foreach($acgroups as $acgroup)
                                        <option value="{{ $acgroup->acsubgrp_id }}" {{ old('acsubgrp_id')==$acgroup->acsubgrp_id ? "selected":"" }}>{{ $acgroup->subgroup }}</option>
                                    @endforeach
                                </select>
                                @error('acsubgrp_id')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                            </div>
                        </div>
                        <div class="col-12 col-sm-4">
                            <div class="form-group">
                                <label for="opening_balance">Opening Balance</label>
                                <input type="text" class="form-control @error('opening_balance') border-danger @enderror" id="opening_balance" name="opening_balance" placeholder="0.00" value="{{ old('opening_balance') }}">
                                @error('opening_balance')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                            </div>
                        </div>
                    </div>
                    <div class="row">
                        <div class="col-12">
                            <div class="form-group">
                                <button type="submit" class="btn btn-success float-right">Save</button>
                            </div>
                        </div>
                    </div>
                </form>
            </div>
        </div>
        <br>
    @endif
    <div class="card">
        <div class="card-body">
            <table class="table table-bordered" id="ledgerTable" width="100%">
                <thead>
                <tr>
                    <th class="no-sort">#</th>
                    <th>Ledger Name</th>
                    <th>Sub Group</th>
                    <th>Opening Balance</th>
                    <th>Dr/Cr</th>
                    <th class="no-sort">Actions</th>
                </tr>
                </thead>
                <tbody>
                </tbody>
            </table>
        </div>
    </div>
    <script src="{{ asset('public/template/plugins/jquery/jquery.min.js') }}"></script>
    <script>
        $(function () {
            var ledgerTable = $("#ledgerTable").DataTable({
                lengthChange: true,
                lengthMenu: [ [10, 50, 100, 500, -1], [10, 50, 100, 500, 'All'],],
                scrollX: true,
                searching: true,
                processing : true,
                serverSide : true,
                stateSave : true,
                columnDefs : [{ targets: 'no-sort', orderable: false }],
                ajax : {
                    url : '{{ route('accounts.ledgers.ajax') }}',
                    dataType : 'json',
                    type : 'GET',
                },
                columns : [
                    { data : 'sno' },
                    { data : 'name' },
                    { data : 'group' },
                    { data : 'balance' },
                    { data : 'dc' },
                    { data : 'action' },
                ]
            });
        });
    </script>
@endsection