@extends('layouts.template')
@section('title','Receipt Voucher')
@section('heading','Receipt Voucher')
@section('breadcrumb','Home / Voucher / Receipt / Update')
@section('content')
    <div class="card card-primary">
        <div class="card-header">Update Receipt Voucher</div>
        <div class="card-body">
            <form action="{{ route('accounts.receipt.voucher.update') }}" method="post">
                @csrf
                <input type="hidden" name="voucher_id" value="{{ encrypt($voucher->id) }}">
                <div class="row">
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="voucher_type" class="required">Voucher Type</label>
                            @php $vtypeid = old('voucher_type') ?? $voucher->vtype_id; @endphp
                            <select class="form-control" id="voucher_type" name="voucher_type" required disabled>
                                @foreach($vtypes as $vtype)
                                    <option value="{{ $vtype->id }}" {{ $vtypeid == $vtype->id ? 'selected' : '' }}>{{ $vtype->vtype_name }}</option>
                                @endforeach
                            </select>
                            @error('voucher_type')<div class="text-danger small mt-1">{{ $message }}</div>@enderror
                        </div>
                    </div>
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="voucher_no" class="required">Voucher No</label>
                            <input type="text" class="form-control @error('voucher_no') border-danger @enderror" id="voucher_no" name="voucher_no" value="{{ $voucher->vno }}" readonly required>
                            @error('voucher_no')<div class="text-danger small mt-1">{{ $message }}</div>@enderror
                        </div>
                    </div>
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="voucher_date" class="required">Voucher Date</label>
                            <input type="date" class="form-control @error('voucher_date') border-danger @enderror" id="voucher_date" name="voucher_date" value="{{ old('voucher_date') ?? $voucher->vdate }}" required>
                            @error('voucher_date')<div class="text-danger small mt-1">{{ $message }}</div>@enderror
                        </div>
                    </div>
                </div>
                <div class="row">
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="debit" class="required">Debit</label>
                            @php $debitid = old('debit') ?? $voucher->dr_ledger; @endphp
                            <select class="js-example-basic-single form-control" id="debit" name="debit" required>
                                <option value="">Select</option>
                                @foreach($ledgers as $ledger)
                                    <option value="{{ $ledger->ledger_id }}" {{ $debitid == $ledger->ledger_id ? 'selected' : '' }}>{{ $ledger->ledger_name }}</option>
                                @endforeach
                            </select>
                            @error('debit')<div class="text-danger small mt-1">{{ $message }}</div>@enderror
                        </div>
                    </div>
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="credit" class="required">Credit</label>
                            @php $creditid = old('credit') ?? $voucher->cr_ledger; @endphp
                            <select class="js-example-basic-single form-control" id="credit" name="credit" required>
                                <option value="">Select</option>
                                @foreach($ledgers->whereNotIn('ag_id', [13, 14]) as $ledger)
                                    <option value="{{ $ledger->ledger_id }}" {{ $creditid == $ledger->ledger_id ? 'selected' : '' }}>{{ $ledger->ledger_name }}</option>
                                @endforeach
                            </select>
                            @error('credit')<div class="text-danger small mt-1">{{ $message }}</div>@enderror
                        </div>
                    </div>
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="amount" class="required">Amount</label>
                            <input type="text" class="form-control @error('amount') border-danger @enderror" id="amount" name="amount" value="{{ old('amount') ?? $voucher->vamount }}" required>
                            @error('amount')<div class="text-danger small mt-1">{{ $message }}</div>@enderror
                        </div>
                    </div>
                </div>
                <div class="row">
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="cdno">Cheque/DD No</label>
                            <input type="text" class="form-control" id="cdno" name="cdno" value="{{ old('cdno') ?? $voucher->cdno }}">
                        </div>
                    </div>
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="cto">To</label>
                            <input type="text" class="form-control" id="cto" name="cto" value="{{ old('cto') ?? $voucher->cto }}">
                        </div>
                    </div>
                    <div class="col-12 col-sm-4">
                        <div class="form-group">
                            <label for="cdate">Cheque Date</label>
                            <input type="date" class="form-control @error('cdate') border-danger @enderror" id="cdate" name="cdate" value="{{ old('cdate') ?? $voucher->cdate }}">
                            @error('cdate')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                        </div>
                    </div>
                </div>
                <div class="row">
                    <div class="col-12">
                        <div class="form-group">
                            <label for="remarks">Remarks</label>
                            <textarea class="form-control" id="remarks" name="remarks" rows="3">{{ old('remarks') ?? $voucher->remarks }}</textarea>
                        </div>
                    </div>
                </div>
                <div class="row">
                    <div class="col-12">
                        <div class="form-group">
                            <button type="submit" class="btn btn-success float-right">Update</button>
                            <a href="{{ url()->previous() }}" class="btn btn-secondary float-right mr-2">Cancel</a>
                        </div>
                    </div>
                </div>
            </form>
        </div>
    </div>
    <!-- Consider moving these scripts to a layout or including them conditionally -->
    <script src="{{ asset('public/template/plugins/jquery/jquery.min.js') }}"></script>
    <script src="{{ asset('public/js/accounts/voucher.js') }}"></script>
@endsection