@extends('layouts.template')
@section('title','Home')
@section('heading','Fee Payment')
@section('breadcrumb','Home/Fees/Fee Payment')
@section('content')
    <div class="card card-primary">
        <div class="card-header">Fee Payment</div>
        <div class="card-body">
            <form action="{{ route('fee.get') }}" method="post">
                @csrf
                <div class="row">
                    <div class="col-md-4">
                        <div class="form-group">
                            <label for="student_id" class="required">Student</label>
                            <select class="js-example-basic-single" id="student_id" name="student_id" style="width:100%">
                                <option value="">Select</option>
                                @foreach($students as $student)
                                    <option value="{{ $student->student_id }}" {{ $student_id == $student->student_id ? "selected":"" }}>{{ $student->adno.' - '.$student->name }}</option>
                                @endforeach
                            </select>
                            @error('student_id')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                        </div>
                    </div>
                    <div class="col-md-3">
                        <div class="form-group">
                            <label for="paid_date" class="required">Date</label>
                            <input type="date" class="form-control @error('paid_date') border-danger @enderror" id="paid_date" name="paid_date" value="{{ $paid_date }}">
                            @error('paid_date')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                        </div>
                    </div>
                    <div class="col-md-5">
                        <div class="form-group">
                            <label for="mode_id" class="required">Payment Mode</label>
                            <div class="row">
                                <div class="col-md-7">
                                    <div class="form-group">
                                        <select class="js-example-basic-single" id="mode_id" name="mode_id" style="width:100%">
                                            @foreach($paymodes as $paymode)
                                            <option value="{{ $paymode->mode_id }}" {{ $mode_id == $paymode->mode_id ? "selected":"" }}>{{ $paymode->mode_name }}</option>
                                            @endforeach
                                        </select>
                                        @error('mode_id')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                                    </div>
                                </div>
                                <div class="col-md-5">
                                    <div class="form-group">
                                        <button type="submit" class="btn btn-sm btn-primary">Get Details</button>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </form>
        </div>
    </div>
    <br>
    @if(!empty($student_det))
        <div class="row">
              <div class="col-12 col-sm-3">
    <div class="card" style="border:solid 1px #EDEADE; border-radius:20px; border-top:3px solid #004170; overflow:hidden;">
        <div class="card-body">
            <div class="form-group text-center">
                <img class="img-md rounded-circle" src="{{ asset('public/uploads/student/photo/'.$student_det->photo) }}" alt="Profile image" width="200px" height="200px">
            </div>

            <div class="form-group text-center mb-1"> 
                <p><b><span style="font-size:18px;">{{ strtoupper($student_det->name) }}</span></b></p>
            </div>

            @foreach($crs as $course)
            <div class="form-group text-center mb-1"> 
                <p><b><span class="bg-success text-dark" style="font-size:15px">Major {{$loop->iteration}}:{{strtoupper($course->course_code)}}</span></b></p>
            </div>
@endforeach

           
                    <div class="form-group text-center mb-1"> <!-- Reduced margin bottom -->
                        <p><b><span style="background:#FEBE10;color:black;font-size:15px">{{$semester->semester_name}}</span></b></p>
                    </div>
              

            <div class="form-group row">
                <div class="col-12 col-sm-6">
                    <p class="font-weight-bold">Adm No</p>
                </div>
                <div class="col-12 col-sm-6">
                    <p>: <b>{{$student_det->adno}}</b></p>
                </div>
            </div>  

            <div class="form-group row">
                <div class="col-12 col-sm-6">
                    <p class="font-weight-bold">Phone</p>
                </div>
                <div class="col-12 col-sm-6">
                    <p>: {{$student_det->mobile}}</p>
                </div>
            </div>  

            <div class="form-group row">
                <div class="col-12 col-sm-6">
                    <p class="font-weight-bold">Ac Year</p>
                </div>
                <div class="col-12 col-sm-6">
                    <p>: {{ date('Y', strtotime($student_det->acyear->sdate)) . '-' . date('Y', strtotime($student_det->acyear->edate)) }}</p>
                </div>
            </div> 

           
        </div>    
    </div>
</div>
            <div class="col-12 col-sm-9">
                <div class="row">
                    <div class="col-12">
                        <div class="accordion bg-warning">Payment History</div>
                        <div class="accordion-content">
                            <div style="padding: 20px 0;">
                            <table class="table table-bordered">
                                <thead>
                                    <tr>
                                        <th><b>Bill Number</b></th>
                                        <th><b>Bill Date</b></th>
                                        <th><b>Fee Amount</b></th>
                                        <th><b>Discount</b></th>
                                        <th><b>Fine</b></th>
                                        <th><b>Payment Mode</b></th>
                                        <th><b>Print</b></th>
                                    </tr>
                                </thead>  
                                <tbody>
                                    @forelse($grouped_payments as $bill_number => $payments)
                                        @php
                                            $total_fee_amount = $payments->sum('fee_amount');
                                            $total_discount = $payments->sum('discount');
                                            $total_fine = $payments->sum('fine_amount');
                                        @endphp
                                        <tr>
                                            <td>{{ $bill_number }}</td>
                                            <td>{{ $payments[0]->paid_date }}</td>
                                            <td>{{ $total_fee_amount - $total_discount + $total_fine }}</td>
                                            <td>{{ $total_discount }}</td>
                                            <td>{{ $total_fine }}</td>
                                            <td>{{ $payments[0]->mode->mode_name }}</td>
                                            <td>
                                                <a href="{{ route('searchstudent.history.print', ['receipt_no' => $payments[0]->receipt_no, 'paid_date' => $payments[0]->paid_date]) }}" target="_blank">
                                                    <i style="font-size:20px" class="icon-sm icon-printer text-danger"></i>
                                                </a>
                                            </td>
                                        </tr>
                                    @empty
                                        <tr>
                                            <td colspan="7">No Result Found!</td>
                                        </tr>
                                    @endforelse
                                </tbody>
                            </table>
                            </div>
                        </div>
                    </div>
                </div>
                <br>
                <div class="card">
                    <div class="card-body">
                        <form action="{{ route('fee.save') }}" method="post">
                            @csrf
                            <input type="hidden" name="sid" value="{{ encrypt($student_id) }}">
                            <input type="hidden" name="pdate" value="{{ $paid_date }}">
                            <input type="hidden" name="mode" value="{{ $mode_id }}">
                            <div class="row">
                                <div class="col-12 col-md-3">
                                    <label for="receipt_no" class="required"><b>Bill Number</b></label>
                                    <div class="form-group">
                                        <input type="text" style="border:1px solid #D3D3D3" class="form-control @error('receipt_no') border-danger @enderror" id="receipt_no" name="receipt_no" value="{{$next_rec_no}}" readonly>
                                        @error('receipt_no')<div class="text-daborder-danger small mt-1 text-danger">{{ $message }}</div>@enderror
                                    </div>
                                </div>
                            </div>
                            <div class="row">
                                <div class="col-12">
                                    <div class="scroll">
                                        <table class="table table-bordered" width="100%">
                                            <thead>
                                                <th width="4%">
                                                    <div class="form-check form-check-success">
                                                        <label class="form-check-label">
                                                            <input type="checkbox" class="form-check-input" id="checkAll"> &nbsp; 
                                                        </label>
                                                    </div>
                                                </th>
                                                <th width="10%">Semester</th>
                                                <th width="6%">Term</th>
                                                <th width="10%">Fee Type</th>
                                                <th width="10%">Fee Amt</th>
                                                <th width="10%">Due Amt</th>
                                                <th width="10%">Fine Payable</th>
                                                <th width="10%">Current Amt</th>
                                                <th width="10%">Total Amt</th>
                                                <th width="10%">Dis.Amt</th>
                                                <th width="10%">Due Date</th>
                                            </thead>
                                            <tbody>
                                                @php 
                                                $termFee_total=0; 
                                                $totalDue_amount=0;
                                                foreach($fees as $ac_id => $sem_details){
                                                    foreach($sem_details as $sem_id => $term_details){
                                                        foreach($term_details['fee_structure'] as $term_id => $fee_details){
                                                            foreach($fee_details['fee_type'] as $ftype_id => $ftype_details){
                                                                $famount = $ftype_details['due_amount']+$ftype_details['fine_amount']; 
                                                                if($famount > 0){
                                                                    $totalDue_amount += $ftype_details['due_amount'];
                                                                }
                                                                $isPaid = ($famount == 0);
                                                                if(!$isPaid || ($isPaid && $ftype_details['term_fee'] > 0)){
                                                                    $termFee_total += $ftype_details['term_fee']; @endphp
                                                                    <tr>
                                                                        <td>
                                                                            <div class="form-check form-check-success d-flex">
                                                                                @if(!$isPaid)
                                                                                    <label class="form-check-label">
                                                                                        <input type="checkbox" class="form-check-input checkSub" name="posted[]" value="{{ $ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}"> &nbsp; 
                                                                                    </label>
                                                                                @endif
                                                                            </div>
                                                                        </td>
                                                                        <td>{{ $term_details['name'] }}</td>
                                                                        <td>{{ $fee_details['name'] }}</td>
                                                                        <td>{{ $ftype_details['ftype_name'] }}</td>
                                                                        <td align="center">{{$ftype_details['term_fee']}}</td>
                                                                        <td align="center">{{ $ftype_details['due_amount'] }}
                                                                            <input type="hidden" id="{{ 'current_fee'. $ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}" value="{{ $ftype_details['due_amount'] }}">
                                                                            <input type="hidden" id="{{ 'current_fine'. $ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}" name="{{ 'current_fine'.$ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}" value="{{ $ftype_details['fine_amount'] }}">
                                                                            <input type="hidden" name="{{ 'fee_type'. $ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}" value="{{ $ftype_id }}">
                                                                        </td>
                                                                        <td align="right"><input type="text" class="form-control text-right" id="{{ 'fine_amount'. $ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}" name="{{ 'fine_amount'.$ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}" placeholder="0.00" value="{{ $ftype_details['fine_amount'] }}" onkeyup="showFine(this.value,'{{ $sem_id }}','{{ $term_id }}','{{ $ftype_id }}')"></td>
                                                                        <td align="right"><input type="text" class="form-control text-right" id="{{ 'fee_amount'. $ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}" name="{{ 'fee_amount'.$ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id }}" placeholder="0.00" value="{{ $ftype_details['due_amount'] }}" onkeyup="showFee(this.value,'{{ $sem_id }}','{{ $term_id }}','{{ $ftype_id }}')"></td>
                                                                        <td align="center">
                                                                            @if($isPaid)
                                                                                <span class="text-success">Paid !!</span>
                                                                            @else
                                                                                {{ $famount }}
                                                                            @endif
                                                                        </td>
                                                                        <td align="right">
                                                                            <input type="text" name="{{'discount'.$ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id}}"  id="{{ 'discount'. $ac_id.'-'.$sem_id.'-'.$term_id.'-'.$ftype_id}}"  class="form-control fee-discount"  placeholder="0.00" onkeyup="showDiscount(this.value,'{{ $sem_id }}','{{ $term_id }}','{{ $ftype_id }}')">
                                                                        </td>
                                                                        <td>{{ date('d-m-Y',strtotime($ftype_details['end_date'])) }}</td>
                                                                    </tr>
                                                                @php }
                                                            } 
                                                        }
                                                    }
                                                } @endphp
                                                <tr>
                                                    <td colspan="4" align="center"><b>Total</b></td>
                                                    <td align="center"><b>{{$termFee_total}}</b></td>
                                                    <td align="center"><b>{{$totalDue_amount}}</b></td>
                                                    <td colspan="1" align="right"><b>Remarks</b></td>
                                                    <td colspan="5"><input type="text" name="remarks" id="textBoxremarks" class="form-control" style="border:1px solid gray"></td>
                                                </tr>
                                            </tbody>              
                                        </table>
                                    </div>
                                </div>
                            </div>
                            <div class="row">
                                <div class="col-md-3 text-right">
                                    <p class="font-weight-bold">Net Amount :</p>
                                </div>
                                <div class="col-md-3">
                                    <span class="font-weight-bold" id="gtotal">0.00</span>
                                </div>
                            </div>
                            <div class="row">
                                <div class="col-md-12">
                                    <div class="form-group">
                                        <button type="submit" class="btn btn-primary float-right">Submit</button>
                                    </div>
                                </div>
                            </div>
                        </form>
                    </div>
                </div>
            </div>
        </div>
    @else
        <div class="alert alert-primary" role="alert">No Result Found!</div>
    @endif
    <script src="{{ asset('public/template/plugins/jquery/jquery.min.js') }}"></script>
    <script src="{{ asset('public/js/fee.js') }}"></script>
    <script>
        document.addEventListener("DOMContentLoaded", function() {
            // Assuming you have a form to validate
            document.querySelector('form').addEventListener('submit', function(event) {
                var modeSelect = document.getElementById('mode_id');
                if (modeSelect.value === '') {
                    alert('Please select a payment mode.');
                    event.preventDefault();
                }
            });
        });
    </script>
@endsection